<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * EcmEngine Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	DataLog
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class EcmEngine extends MX_Controller {
   private $accessTime;
   private $mode;
   
   /**
	* Holder for get string
	*
	* @var array
	* @access private
	*/
   private $get;
	
  /**
   * System settings
   *
   * @var object
   * @access private
   */
  private $sysinfo;
   
  /**
   * Constructor
   *
   * @access	public
   */
  function EcmEngine()
  {
    parent::__construct();
	    
	// Load models
    $this->load->model('sysinfo_model');
    $this->load->model('channels_model');
    $this->load->model('device_model');
    $this->load->model('addon_model');
	$this->load->model('commands_model');
	$this->load->model('minute_model');
	
  	$this->sysinfo = $this->sysinfo_model->getInfo();
	
	// If not setup, kill the data logger
	if($this->sysinfo->num_rows() == 0)
	{
		die;
	}
	
    $this->sysinfo = $this->sysinfo->row_array();
	
	date_default_timezone_set($this->sysinfo["timezone"]);
    
	// Time for raw data log
	$this->accessTime = date('Y-m-d H:i:s'); 
	
	
    parse_str($_SERVER['QUERY_STRING'], $this->get);
	
	// Check if raw data log too large, if so overwrite it
	$this->mode = 'a';
	if(file_exists('data'))
	{
		if(filesize('data') > 10000)
		{
		 	$this->mode = 'w';
		}
	}
	$this->db->query("ANALYZE TABLE minutedata;");
	// In case of interrupted packets
    ignore_user_abort(true); 
  }

   // --------------------------------------------------------------------

   /**
	* Process ECM-1240 packet
	*
	* @access	public
	* @return	void
	*/
  function ecm1240()
  {
  	// Log the query string
	$ch = fopen('data', $this->mode);
	fwrite($ch,$this->accessTime.': '.$_SERVER['QUERY_STRING']."<br>");
	fclose($ch);
	
    $check = true;
	// Check the values in the query string
    if(isset($this->get['sec']) && isset($this->get['v']) && isset($this->get['wsa1']) && isset($this->get['wsa2']) && isset($this->get['A1ws']) && 
          isset($this->get['A2ws']) && isset($this->get['A3ws']) && isset($this->get['A4ws']) && isset($this->get['A5ws']) && isset($this->get['dev'])
           && isset($this->get['id']) && isset($this->get['wsap1']) && isset($this->get['wsap2']))
    {
      $check = $this->_checkVal($this->get['sec']);
      $check = $this->_checkVal($this->get['v']);
      $check = $this->_checkVal($this->get['wsa1']);
      $check = $this->_checkVal($this->get['wsa2']);
      $check = $this->_checkVal($this->get['A1ws']);
      $check = $this->_checkVal($this->get['A2ws']);
      $check = $this->_checkVal($this->get['A3ws']);
      $check = $this->_checkVal($this->get['A4ws']);
      $check = $this->_checkVal($this->get['A5ws']);
      $check = $this->_checkVal($this->get['dev']);
      $check = $this->_checkVal($this->get['id']);
      $check = $this->_checkPolarized($this->get['wsap1']);
      $check = $this->_checkPolarized($this->get['wsap2']);
	  
	  
	  // Serial number check
      if($this->get['dev'] == "")
      {
        $check = false;
      }
      if($this->get['id'] == "")
      {
        $check = false;
      }
	  
      if($check == false)
	  {
		  $ch = fopen('data', 'a');
		  fwrite($ch,"Error in ECM-1240 packet<br>");
		  fclose($ch);
	  }

      if($check == true)
      {
      	
      	// If any fields are null, set them to 0
        $sec = ($this->get['sec'] == "") ? 0 : $this->get['sec'];
        $v = ($this->get['v'] == "") ? 0 : $this->get['v'];
        $ch1ws = ($this->get['wsa1'] == "") ? 0 : $this->get['wsa1'];
        $ch2ws = ($this->get['wsa2'] == "") ? 0 : $this->get['wsa2'];
        $aux1ws = ($this->get['A1ws'] == "") ? 0 : $this->get['A1ws'];
        $aux2ws = ($this->get['A2ws'] == "") ? 0 : $this->get['A2ws'];
        $aux3ws = ($this->get['A3ws'] == "") ? 0 : $this->get['A3ws'];
        $aux4ws = ($this->get['A4ws'] == "") ? 0 : $this->get['A4ws'];
        $aux5ws = ($this->get['A5ws'] == "") ? 0 : $this->get['A5ws'];
        $dev = $this->get['dev'];
        $id = $this->get['id'];
        $ch1pol = ($this->get['wsap1'] == "") ? 0 : $this->get['wsap1'];
        $ch2pol = ($this->get['wsap2'] == "") ? 0 : $this->get['wsap2'];
        
		// Voltage conversion
        $vDec = substr($v, -1);
        $v = substr($v, 0, -1).'.'.$vDec;
        
		// Create wattseconds and polarized arrays
        $wsArray = array($ch1ws,$ch2ws,$aux1ws,$aux2ws,$aux3ws,$aux4ws,$aux5ws);
        $polArray = array($ch1pol, $ch2pol);
        
		// Convert the serial number
        if(strlen($dev) < 5)
        {
          $cnt = 5 - strlen($dev);
          for($x = 0; $x<$cnt; $x++)
          {
            $dev = "0".$dev;
          }
        }
        $dev = $id.$dev;
		
		// Run the data logger
        modules::run('datalog/log1240', $wsArray, $polArray, $v, $dev, $sec, 7);
		
		// Run the addons
	  	$device = $this->device_model->getDevice($dev);
		if($device->num_rows() > 0)
		{
			$this->_runAddons($dev, 8);
		}
      }
    }
  }
	
  // --------------------------------------------------------------------

   /**
	* Process GreenEye packet
	*
	* @access	public
	* @return	void
	*/
  function greeneye()
  {	
	// First bad packet check, if these values aren't set it's an obvious bad packet
	if(!isset($this->get['SC']) || !isset($this->get['SN']) || !isset($this->get['V']))
	{
		$ch = fopen('data', $this->mode);
		fwrite($ch,"Bad Packet Missing Voltage, SN, or SC<br>");
		fclose($ch);
		die;	
	}
	
	// Log to raw data log
	$ch = fopen('data', $this->mode);
	fwrite($ch,$this->accessTime.':  Serial Number is '.$this->get['SN']." and seconds is  ".$this->get['SC']."<br>");
	fclose($ch);
	
	// Holder for channel size, 48 in the future
	$chAmt = 32;
	
	$check = true;
	
	$wsArray = array();
	$polArray = array();
	
	
	
	for($x=1;$x<=$chAmt;$x++)
	{
		// Check for problems with each channel
		if(!isset($this->get['c'.$x]))
		{
			$check = false;
			$ch = fopen('data', 'a');
			fwrite($ch,"Bad Packet Missing Channel ".$x."<br>");
			fclose($ch);
			break;
		}
		
		// Get wattseconds and polarized
		$tmp = explode(',', $this->get['c'.$x]);
		
		// Check wattseconds
		if(! $this->_checkVal($tmp[0]))
		{
			$check = false;
			$ch = fopen('data', 'a');
			fwrite($ch,"Bad Packet Not Numeric Channel ".$x."<br>");
			fclose($ch);
			break;
		}
		
		$wsArray[$x-1] = $tmp[0];	
		
		// Check polarized
		if(sizeof($tmp) > 1)
		{
			if(! $this->_checkVal($tmp[1])) 
			{
				$check = false;
				$ch = fopen('data', 'a');
				fwrite($ch,"Bad Packet Not Numeric Channel ".$x."<br>");
				fclose($ch);
				break;
			}
			
		    $polArray[$x-1] = $tmp[1];
		}
	}

	
    if($check)
    {
	  // Create arrays for pulse and temp
	  $pulse = explode(',', $this->get['PL']);
	  $temp = explode(',', $this->get['T']);
      
	  // Convert voltage
      $vDec = substr($this->get['V'], -1);
      $v = substr($this->get['V'], 0, -1).'.'.$vDec;
	  
	  // Log the values
	  $ch = fopen('data', $this->mode);
	  fwrite($ch,$this->get['SN']." Debug<br>Pulse: ".$this->get['PL']."<br>Wattseconds: ".implode(",", $pulse)."<br>"."Temps: ".$this->get['T']."<br>-------------<br>");
	  fclose($ch);
	  
	  // Call the datalogger
      modules::run('datalog/loggeye', $wsArray, $polArray, $v, $this->get['SN'], $this->get['SC'], $chAmt, $pulse, $temp);
   	  
	  // Run the addons
  	  $device = $this->device_model->getDevice($this->get['SN']);
	  if($device->num_rows() > 0)
	  {
		  $this->_runAddons($this->get['SN'], 33);
	  }
    }
  }
  
  
  // --------------------------------------------------------------------

  /**
   * Process Add-on Services
   *
   * @access	private
   * @return	void
   */
  private function _runAddons($dev, $w = 33)
  {
  	$this->_runSEG($dev, $w);
  }
  
  // --------------------------------------------------------------------

  /**
   * SmartEnergyGroups Add-on Processor
   *
   * @access	private
   * @return	void
   */
  private function _runSEG($serial, $w = 33)
  {
  	// Get the users SEG information
	$addon = $this->addon_model->GetAddon("SmartEnergyGroups");
	
	// If the add-on exists, continue
	if($addon->num_rows() > 0)
	{
		$addon = $addon->row_array();
		
		
		// Check if the add-on is enabled  
	  	$enabled = $this->addon_model->GetAddonVar("enabled", $addon['addon_id']);
	
		if($enabled->num_rows() > 0)
	    {
			$enabled = $enabled->row_array();
			
			if($enabled['value'] == "TRUE")
			{
					
				// Get the SEG Key
			  	$key = $this->addon_model->GetAddonVar("key", $addon['addon_id']);
				$key = $key->row_array();
				
		  	 	$device = $this->device_model->getDevice($serial);
				$device = $device->row_array();
				
				
				// Get the date last posted
				$date = $this->addon_model->GetAddonVar("date".$device['device_id'], $addon['addon_id']);
				if($date->num_rows() > 0)
				{
					$date = $date->row_array();
					
					if($date['value'] < strtotime($device['mindate']))
					{
				  	 	$channels = $this->channels_model->getDeviceChannels($device['device_id']);
						
						
						// Update the last post date
						$this->addon_model->UpdateAddonVar($addon['addon_id'], 'date'.$device['device_id'], strtotime($device['mindate']));
						
						
						// Start of the SEG cURL string
					    $curlStr = "(site ".$key['value']." (node ".$serial." ? ";
						
						$x = 1;
						$y = 1;
						$z = 1;
						
						foreach($channels->result() as $channel)
						{
							
							// Grab the last reading for each channel and append it to the SEG string
							$power = $this->minute_model->getLastReading($channel->channel_id);
							
							if($power->num_rows() > 0)
							{
								$power = $power->row_array();
								if($channel->ctype == 0 && $channel->netchannel_id == 0) // If normal channel
								{
									$curlStr .=  "(p_".$x." ".($power['watt']).")(e_".$x." ".(round($power['kwh']*1000,2)).")";
									$x++;	
								}
								else if($channel->ctype == 0 && $channel->netchannel_id > 0)  // If NET Metered channel
								{
									$curlStr .=  "(p_".$w." ".($power['watt']).")(e_".$w." ".(round($power['kwh']*1000,2)).")";
									$w++;
								}
								else if($channel->ctype == 1)
								{
									$curlStr .=  "(temperature_".$y." ".round($power['kwh'],2).")";
									$y++;
								}
								else if($channel->ctype == 2)
								{
									$curlStr .=  "(n_".$z." ".round($power['kwh'],2).")";
									$z++;
								}
								else if($channel->ctype == 3)
								{
									$curlStr .=  "(v_1 ".round($power['kwh'],2).")";
								}
							}
							else
							{
								// If no record, just increment the counters
								if($channel->ctype == 0 && $channel->netchannel_id == 0)
								{
									$y++;
								}
								else if($channel->ctype == 0 && $channel->netchannel_id > 0)
								{
									$w++;
								}
								else if($channel->ctype == 2)
								{
									$z++;
								}
								else if($channel->ctype == 0)
								{
									$x++;	
								}
							}
							
							
						}
						
						// Finish the string then post it to SEG
						$curlStr .= '))';
						
						$params = array('http' => array(
						                'method' => 'PUT',
						                'header' => "Content-Type: application/x-www-form-urlencoded\r\n",
										'content' => http_build_query(array("data_post" => $curlStr))));
						 
						 
						$ctx = stream_context_create($params);
						$fp = @fopen('http://api.smartenergygroups.com/api_sites/stream', 'rb',false, $ctx);
						
						// Send the post to the data logger
						$ch = fopen('data', 'a');
						fwrite($ch, date('Y-m-d h:i:s A').' <b>SmartEneryGroups</b> '.$curlStr."<br>");
						fclose($ch);
					}
				}
				else
				{
					// If no date, insert the current date
					$this->addon_model->InsertAddonVar($addon['addon_id'], 'date'.$device['device_id'], strtotime($device['mindate']));
				}
			}
		 }
	 }
  }

  // --------------------------------------------------------------------

  /**
   * Check for valid or none
   *
   * @access	public
   * @return	boolean
   */
  private function _checkVal($val)
  {
    if($val != "")
    {
      if(is_numeric($val))
      {
        return true;
      }
    }
    return false;
  }
  
  // --------------------------------------------------------------------

  /**
   * Check polarized values
   *
   * @access	private
   * @return	boolean
   */
  private function _checkPolarized($val)
  {
    if($val != "")
    {
      if(!is_numeric($val))
      {
        return false;
      }
    }
    return true;
  }
  
}

/* End of file ecmEngine.php */
/* Location: ./btech/controllers/ecmEngine.php */
