<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * Device Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	Devices
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class Device extends MX_Controller {
   
  /**
   * Constructor
   *
   * @access	public
   */
  function Device()
  {
    parent::__construct();
    $this->load->library('form_validation');
    $this->load->helper('form');
    $this->load->helper('url');
    $this->load->model('device_model');
    $this->load->model('channels_model');
	$this->load->library('session');
	if($_SERVER['REMOTE_ADDR'] != "::1")
	{
	    if (!$this->session->userdata('admin'))
	    {
	        redirect('/pages/dashboard/');
	    }
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Add a new device
	*
	* @access	public
	* @return	view
	*/
  function AddDevice()
  {
  	$this->form_validation->set_rules('serial','Device Serial','min_length[5]|max_length[8]|required');
  	$this->form_validation->set_rules('type','Device Type','max_length[8]|required');
	if($this->form_validation->run() == FALSE)
    {
       $this->load->view('home/settings/adddevice');  
	}
	else
	{
		$this->device_model->InsertDevice(set_value('serial'),set_value('type'));
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Remove a device
	*
	* @access	public
	* @return	view
	*/
  function RemoveDevice()
  {
  	$this->form_validation->set_rules('serial','Device Serial','min_length[2]|max_length[8]|required');
	
	if($this->form_validation->run() == FALSE)
    {
       $data['devices'] = $this->device_model->getDevices('virtual',1);
       $this->load->view('home/settings/removedevice', $data);  
	}
	else
	{
		$this->device_model->DeleteDevice(set_value('serial'));
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Update an ECM-1240s Settings
	*
	* @access	public
	* @return	view
	*/
  function ecm1240($device, $msg = 0)
  {
    $dev = $this->device_model->getDevice($device);
    $dev = $dev->row_array();
    
	// Expected values
    $this->form_validation->set_rules('alias','Device Alias','max_length[25]');
    
    $y = 1;
	// 7 channels
    for($x=1;$x<8;$x++)
    {
      $this->form_validation->set_rules('channel'.$x,'Channel ID');
      $this->form_validation->set_rules('main'.$x,'main'.$x,'max_length[1]');
      $this->form_validation->set_rules('ch'.$x.'alias','Channel '.$x.' Alias','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'colour','Channel '.$x.' Colour','max_length[6]');
      $this->form_validation->set_rules('on'.$x,'Channel '.$x.' On Value','max_length[6]|numeric');
    }
    
	// CH1 and 2 can be net
    $this->form_validation->set_rules('net1','Net Channel 1','max_length[1]');
	$this->form_validation->set_rules('net2','Net Channel 2','max_length[1]');
	
	// One pulse counter
    $this->form_validation->set_rules('pulse','pulse','max_length[1]');
    $this->form_validation->set_rules('pulseValue','Pulse Value','callback_pulseValue_check');
    $this->form_validation->set_rules('pulseUnit','Pulse Unit','max_length[8]');
    $this->form_validation->set_rules('device_id','device_id','required');
	
    $this->form_validation->set_rules('serial','Serial Number','max_length[7]|min_length[4]|required');
    
    if($this->form_validation->run() == FALSE)
    {
      $channel = $this->channels_model->getECMChannels($dev['device_id']);
      
	  // Information for view
      $data['device_id'] = $dev['device_id'];
      $data['form'] = "ecm1240";
      $data['serial'] = $dev['serial'];
      $data['alias'] = $dev['alias'];
      $data['chalias'] = array();
      $data['chmain'] = array();
      $data['chpulse'] = array();
      $data['channels'] = '<tr>
			                <th>Channel</th>
			                <th>Main</th>
			                <th>Net</th>
			                <th>Pulse</th>
			                <th>Hide</th>
			                <th>Colour</th>
			                <th>Image</th>
			                <th>Threshold</th>
			              </tr>';
	  $data['msg'] =  $msg;
	  $data['img'] = $this->session->flashdata('img');
	  
      $pulseVal = "";
	  $pulseUnit = "";
      $x = 1;
	  
	  // Loop through all channels and build a table to modify values based on them
      foreach($channel->result() as $row)
      {
      	
        $data['channels'] .= '<input type="hidden" name="channel'.$x.'" value="'.set_value('channel'.$x, $row->channel_id).'"><tr><td>
                <label for="channel">'.$x.'. </label> 
                <input type="text" class="text" name="ch'.$x.'alias" value="'.set_value('ch'.$x.'alias', $row->channel_name).'"></td>';
				
        
        $data['channels'] .=  '<td class="center"><input type="checkbox" id="main" name="main'.$x.'" value="1" ';
        if($row->main == 1)
        {
          $data['channels'] .= set_checkbox('main'.$x, $row->main, TRUE);
        }
        else
        {
          $data['channels'] .= set_checkbox('main'.$x, $row->main);
        }
        $data['channels'] .= ' /></td>';
		
        if($x == 1 || $x == 2)
        {
		  
		  $data['channels'] .=  '<td class="center"><input type="checkbox" id="net" name="net'.$x.'" value="1" ';
          if($this->channels_model->getChannelsNet($row->channel_id)->num_rows() > 0)
          {
            $data['channels'] .= set_checkbox('net'.$x, 1, TRUE);
          }
          else
          {
            $data['channels'] .= set_checkbox('net'.$x, 0);
          }
          $data['channels'] .= ' /></td>';
        }
        else
        {
           $data['channels'] .=  '<td></td>';
        }
        
        if($x == 7)
        {
          $data['channels'] .=  '<td class="center"><input type="checkbox" id="pulse" name="pulse" value="1" ';
		  $pulseVal = $row->pulse;
		  $pulseUnit = $row->pulse_unit;
          if($row->pulse>0)
          {
            $data['channels'] .= set_checkbox('pulse', $row->pulse, TRUE);
          }
          else
          {
            $data['channels'] .= set_checkbox('pulse', $row->pulse);
          }
          $data['channels'] .= ' /></td>';
        }
        else
        {
           $data['channels'] .=  '<td></td>';
        }
		$data['channels'] .=  '<td><input type="checkbox" id="hide" name="hide'.$x.'" value="1" ';
        if($row->hide == 1)
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide);
        }
		$data['channels'] .=  ' /></td>';
  		$data['channels'] .=  '<td><div><input type="text" class="text" maxlength="6" size="6" id="ch'.$x.'colour" name="ch'.$x.'colour" value="'.set_value('ch'.$x.'colour', $row->colour).'" /></div></td>';
		$data['channels'] .= '<td><a href="'.base_url().'index.php/displayimages/Main/1/'.$row->channel_id.'/'.$device.'">Select</a></td>';
	    $data['channels'] .= '<td><input type="text" class="text" id="on'.$x.'" maxlength="6" size="6" name="on'.$x.'" value="'.set_value('on'.$x, $row->views).'" /></td>';
        
        $data['channels'] .=  '</tr>';
        
        if($x == 1)
        {
          $data['colours'] = '#ch'.$x.'colour';
        }
        else
        {
          $data['colours'] .= ', #ch'.$x.'colour';
        }
        $x = $x + 1;
      }
	   
	  if($x > 1)
	  {
	  	$data['channels'] .= '<tr><td class="center" colspan="7"><b>1 Pulse =</b> <input type="text" class="text" value="'.$pulseVal.'" name="pulseValue" size="10"/>&nbsp;<b>Unit:</b>&nbsp;<input value="'.$pulseUnit.'" type="text" name="pulseUnit" size="5"/></td></tr>';
	  }
      $this->load->view('home/settings/dev', $data);  
    }
    else
    {
      // Update the channel values
      if(set_value('channel1') != '')
	  {
	      for($x=1;$x<8;$x++)
	      {
	      	// Update main, hidden, and on threshold values
			$main = 1;
			$hide = 1;
			if(set_value('main'.$x) == '')
			{
				$main = 0;
			}
			  
			if(set_value('hide'.$x) == '')
			{
				$hide = 0;
			}
			  
			if(set_value('on'.$x) == '')
			{
			   $views = 0;
			}
			else
			{
			  	$views = set_value('on'.$x);
			}
			
	        if($x < 7)
	        {
	          if($x < 3)
			  {
			  	  // If net is set, check if it exists and if not add it
		          if(set_value('net'.$x) == 1)
				  {
				  	     if($this->channels_model->getChannelsNet(set_value('channel'.$x))->num_rows() == 0)
						 {
							 $this->channels_model->InsertNet(set_value('channel'.$x), set_value('device_id'), set_value('ch'.$x.'alias')." Gen", 0, $x, $this->_random_colour());
						 }    
				  } 
				  else  // If not set, check if exists and delete if so
				  {
				  	 if($this->channels_model->getChannelsNet(set_value('channel'.$x))->num_rows() > 0)
					 {
						 $this->channels_model->RemoveNet(set_value('channel'.$x));
						 $this->channels_model->RemoveNet(set_value('channel1'));
						 $this->channels_model->RemoveNet(set_value('channel2'));
					 }   
				  }
			  }
			  
			  $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
	                             'colour' => set_value('ch'.$x.'colour'),
	                             'main' => $main,
	                             'views' => $views,
	                             'hide' => $hide,
	                             'pulse' => 0);
	        }
	        elseif($x == 7)
	        {			  
	          if(set_value('pulse') == 1)
			  {
		          $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
		                           'colour' => set_value('ch'.$x.'colour'),
		                           'main' => 0,
		                           'pulse' => set_value('pulseValue'),
	                               'views' => $views,
	                               'hide' => $hide,
	                               'ctype' => 2,
								   'pulse_unit' => set_value('pulseUnit'));
	           }
			  else
			  {
			  	  $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
		                           'colour' => set_value('ch'.$x.'colour'),
		                           'main' => $main,
		                           'pulse' => 0,
	                               'views' => $views,
	                               'hide' => $hide,
	                               'ctype' => 0,
								   'pulse_unit' => '');
			  }
	        }
	        $this->channels_model->UpdateChannel(set_value('channel'.$x),$chUpdate);
	     }
      }
	  
	  // Update the device
	  $net = 1;
	  if(set_value('net') == '')
	  {
	  	$net = 0;
	  }
      
		
      $devUpdate = array( 'alias' => set_value('alias'),
                          'net' => $net,
                          'serial' => set_value('serial'));
						  
      $this->device_model->UpdateDevice($devUpdate,set_value('device_id'));
      
      redirect('/pages/device/ecm1240/'.set_value('serial').'/1');
    }
  }
	
   // --------------------------------------------------------------------

   /**
	* Update a GreenEye Monitors Settings
	*
	* @access	public
	* @return	void
    */
  function geye($device, $msg = 0)
  {
    $dev = $this->device_model->getDevice($device);
    $dev = $dev->row_array();
    
    $this->form_validation->set_rules('alias','Device Alias','max_length[25]');
    $this->form_validation->set_rules('device_id','device_id','required');
    $this->form_validation->set_rules('serial','Serial Number','max_length[8]|min_length[5]|required');
    
	$channel = $this->channels_model->getNoNetChannels($dev['device_id']);
	
	$netCh = $this->channels_model->getNetDevice($dev['device_id']);
	
	$pulseCh = $this->channels_model->getGreenPulses($dev['device_id']);
		
	$tempCh = $this->channels_model->getTemp($dev['device_id']);
    
    $y = 1;
	// Add the channels
    for($x=1;$x<=$channel->num_rows();$x++)
    {
      $this->form_validation->set_rules('channel'.$x,'Channel ID');
      $this->form_validation->set_rules('main'.$x,'main'.$x,'max_length[1]');
      $this->form_validation->set_rules('hide'.$x,'hide'.$x,'max_length[1]');
	  $this->form_validation->set_rules('net'.$x,'Net Channel 1','max_length[1]');
      $this->form_validation->set_rules('ch'.$x.'alias','Channel '.$x.' Alias','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'colour','Channel '.$x.' Colour','max_length[6]');
      $this->form_validation->set_rules('on'.$x,'Channel '.$x.' On Value','max_length[6]|numeric');
    }
	
	// Add the channels, every channel can possibly be net
    for($x=$channel->num_rows()+1;
    	$x<=$channel->num_rows()+$netCh->num_rows();
    	$x++)
    {
      $this->form_validation->set_rules('channel'.$x,'Channel ID');
      $this->form_validation->set_rules('hide'.$x,'hide'.$x,'max_length[1]');
      $this->form_validation->set_rules('main'.$x,'main'.$x,'max_length[1]');
      $this->form_validation->set_rules('ch'.$x.'alias','Channel '.$x.' Alias','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'colour','Channel '.$x.' Colour','max_length[6]');
      $this->form_validation->set_rules('on'.$x,'Channel '.$x.' On Value','max_length[6]|numeric');
    }
	
	// Up to 4 pulse channels
	for($x=$channel->num_rows()+$netCh->num_rows()+1;
    	$x<=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows();
    	$x++)
    {
      $this->form_validation->set_rules('channel'.$x,'Channel ID');
      $this->form_validation->set_rules('hide'.$x,'hide'.$x,'max_length[1]');
      $this->form_validation->set_rules('pulse'.$x,'Channel '.$x.' Pulse','max_length[25]|numeric|required');
      $this->form_validation->set_rules('punit'.$x,'Channel '.$x.' Pulse Unit','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'alias','Channel '.$x.' Alias','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'colour','Channel '.$x.' Colour','max_length[6]');
      $this->form_validation->set_rules('on'.$x,'Channel '.$x.' On Value','max_length[6]|numeric');
    }
	
	// Up to 8 temperate channels
    for($x=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows()+1;
    	$x<=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows()+$tempCh->num_rows();
    	$x++)
    {
      $this->form_validation->set_rules('channel'.$x,'Channel ID');
      $this->form_validation->set_rules('hide'.$x,'hide'.$x,'max_length[1]');
      $this->form_validation->set_rules('ch'.$x.'alias','Channel '.$x.' Alias','max_length[25]');
      $this->form_validation->set_rules('ch'.$x.'colour','Channel '.$x.' Colour','max_length[6]');
      $this->form_validation->set_rules('on'.$x,'Channel '.$x.' On Value','max_length[6]|numeric');
      $this->form_validation->set_rules('tmp'.$x.'Unit','max_length[1]|required');
    }

    $this->form_validation->set_rules('device_id','device_id','required');
    
    
    if($this->form_validation->run() == FALSE)
    {
      
	  // Values for view
      $data['device_id'] = $dev['device_id'];
	  
      $data['form'] = 'geye';
      $data['serial'] = $dev['serial'];
      $data['alias'] = $dev['alias'];
      $data['chalias'] = array();
      $data['chmain'] = array();
      $data['chpulse'] = array();
      $data['channels'] = '<tr>
			                <th>Channel</th>
			                <th>Main</th>
			                <th>Net</th>
			                <th>Hide</th>
			                <th>Colour</th>
			                <th>Image</th>
			                <th>Threshold</th>
			              </tr>';
	  $data['msg'] =  $msg;
	  $data['img'] = $this->session->flashdata('img');
      
      $x = 1;
	  // Create a table for all the channels values
      foreach($channel->result() as $row)
      {
        $data['channels'] .= '<input type="hidden" name="channel'.$x.'" value="'.set_value('channel'.$x, $row->channel_id).'"><tr><td>
                <label for="channel">'.$x.'. </label> 
                <input type="text" class="text"  class="text" name="ch'.$x.'alias" value="'.set_value('ch'.$x.'alias', $row->channel_name).'"></td>';
				
        $data['channels'] .=  '<td class="center"><input type="checkbox" id="main" name="main'.$x.'" value="1" ';
        if($row->main == 1)
        {
           $data['channels'] .= set_checkbox('main'.$x, $row->main, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('main'.$x, $row->main);
        }
        $data['channels'] .= ' /></td>';
		  
        $data['channels'] .=  '<td class="center"><input type="checkbox" id="net" name="net'.$x.'" value="1" ';
        if($this->channels_model->getChannelsNet($row->channel_id)->num_rows() > 0)
        {
           $data['channels'] .= set_checkbox('net'.$x, 1, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('net'.$x, 0);
        }
        $data['channels'] .= ' /></td>';
	   
	    $data['channels'] .=  '<td class="center"><input type="checkbox" id="hide" name="hide'.$x.'" value="1" ';
        if($row->hide == 1)
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide);
        }
	   
        $data['channels'] .= ' /></td>';
        $data['channels'] .=  '<td><div><input type="text" class="text" maxlength="6" size="6" id="ch'.$x.'colour" name="ch'.$x.'colour" value="'.set_value('ch'.$x.'colour', $row->colour).'" /></div></td>';
	    $data['channels'] .= '<td><a href="'.base_url().'index.php/displayimages/Main/4/'.$row->channel_id.'/'.$device.'">Select</a></td>';
	    $data['channels'] .= '<td><input type="text" class="text" id="on'.$x.'" maxlength="6" size="6" name="on'.$x.'" value="'.set_value('on'.$x, $row->views).'" /></td>';
        
        $data['channels'] .=  '</tr>';
        
        if($x == 1)
        {
           $data['colours'] = '#ch'.$x.'colour';
        }
        else
        {
           $data['colours'] .= ', #ch'.$x.'colour';
        }
        $x = $x + 1;
      }
		$data['channels'] .= '<tr><td style="padding-top:10px; font-weight:bold;">Net</td></tr>';
	  
      foreach($netCh->result() as $row)
      {
      	$data['channels'] .= '<input type="hidden" name="channel'.$x.'" value="'.set_value('channel'.$x, $row->channel_id).'"><tr><td>
                <label for="channel">'.$x.'. </label> 
                <input type="text" class="text" name="ch'.$x.'alias" value="'.set_value('ch'.$x.'alias', $row->channel_name).'"></td>';
						
        $data['channels'] .=  '<td class="center"><input type="checkbox" id="main'.$x.'" name="main'.$x.'" value="1" ';
    
        if($row->main == 1)
        {
           $data['channels'] .= set_checkbox('main'.$x, $row->main, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('main'.$x, $row->main);
        }
        $data['channels'] .= ' /></td>';
	   
	    $data['channels'] .=  '<td></td><td class="center"><input type="checkbox" id="hide'.$x.'" name="hide'.$x.'" value="1" ';
        if($row->hide == 1)
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide);
        }
	   
        $data['channels'] .= ' /></td>';
        $data['channels'] .=  '<td><div><input type="text" class="text" maxlength="6" size="6" id="ch'.$x.'colour" name="ch'.$x.'colour" value="'.set_value('ch'.$x.'colour', $row->colour).'" /></div></td>';
	    $data['channels'] .= '<td><a href="'.base_url().'index.php/displayimages/Main/4/'.$row->channel_id.'/'.$device.'">Select</a></td>';
	    $data['channels'] .= '<td><input type="text" class="text" id="on'.$x.'" maxlength="6" size="6" name="on'.$x.'" value="'.set_value('on'.$x, $row->views).'" /></td>';
        
        $data['channels'] .=  '</tr>';
        
        if($x == 1)
        {
           $data['colours'] = '#ch'.$x.'colour';
        }
        else
        {
           $data['colours'] .= ', #ch'.$x.'colour';
        }
        $x = $x + 1;
	  }
		$data['channels'] .= '<tr><td style="padding-top:10px; font-weight:bold;">Pulse</td></tr>';
		
      foreach($pulseCh->result() as $row)
      {
      	$data['channels'] .= '<input type="hidden" name="channel'.$x.'" value="'.set_value('channel'.$x, $row->channel_id).'"><tr><td>
                <label for="channel">'.$x.'. </label> 
                <input type="text" class="text" name="ch'.$x.'alias" value="'.set_value('ch'.$x.'alias', $row->channel_name).'"></td>';
				
        $data['channels'] .=  '<td class="center"></td><td class="center"></td>';
	   
	    $data['channels'] .=  '<td class="center"><input type="checkbox" id="hide" name="hide'.$x.'" value="1" ';
        if($row->hide == 1)
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide);
        }
	   
        $data['channels'] .= ' /></td>';
        $data['channels'] .=  '<td><div><input type="text" class="text" maxlength="6" size="6" id="ch'.$x.'colour" name="ch'.$x.'colour" value="'.set_value('ch'.$x.'colour', $row->colour).'" /></div></td>';
	    $data['channels'] .= '<td><a href="'.base_url().'index.php/displayimages/Main/4/'.$row->channel_id.'/'.$device.'">Select</a></td>';
	    $data['channels'] .= '<td><input type="text" class="text" id="on'.$x.'" maxlength="6" size="6" name="on'.$x.'" value="'.set_value('on'.$x, $row->views).'" /></td>';
        
        $data['channels'] .=  '</tr>';
        
        if($x == 1)
        {
           $data['colours'] = '#ch'.$x.'colour';
        }
        else
        {
           $data['colours'] .= ', #ch'.$x.'colour';
        }
		$data['channels'] .= '<tr><td colspan="7" style="padding-left:30px;"><b>1 Pulse</b> = <input type="text" class="text" size="3" name="pulse'.$x.'"value="'.set_value('pulse'.$x, $row->pulse).'" /><b>&nbsp;Unit:</b> <input type="text" class="text" style="width:70px;" name="punit'.$x.'" value="'.set_value('punit'.$x, $row->pulse_unit).'" /><td></tr>';
		
        $x = $x + 1;
	  }
		$data['channels'] .= '<tr><td style="padding-top:10px; font-weight:bold;">Temperature</td></tr>';
	  
      foreach($tempCh->result() as $row)
      {
      	$data['channels'] .= '<input type="hidden" name="channel'.$x.'" value="'.set_value('channel'.$x, $row->channel_id).'"><tr><td>
                <label for="channel">'.$x.'. </label> 
                <input type="text" class="text" name="ch'.$x.'alias" value="'.set_value('ch'.$x.'alias', $row->channel_name).'"></td>';
				
        $data['channels'] .=  '<td class="center">Unit:</td><td class="center"><select name="tmp'.$x.'Unit"><option value="C" '; 
        
        if(set_value('tmp'.$x.'Unit', $row->pulse_unit) == "C") $data['channels'] .=  'selected="selected"';
        
        $data['channels'] .=  ' >C</option><option value="F" ';
        
        if(set_value('tmp'.$x.'Unit', $row->pulse_unit) == "F") $data['channels'] .=  'selected="selected"';
        
        $data['channels'] .= ' >F</option></select></td>';
	   
	    $data['channels'] .=  '<td class="center"><input type="checkbox" id="hide" name="hide'.$x.'" value="1" ';
        if($row->hide == 1)
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide, TRUE);
        }
        else
        {
           $data['channels'] .= set_checkbox('hide'.$x, $row->hide);
        }
	   
        $data['channels'] .= ' /></td>';
        $data['channels'] .=  '<td><input type="text" class="text" maxlength="6" size="6" id="ch'.$x.'colour" name="ch'.$x.'colour" value="'.set_value('ch'.$x.'colour', $row->colour).'" /></td>';
	    $data['channels'] .= '<td><a href="'.base_url().'index.php/displayimages/Main/4/'.$row->channel_id.'/'.$device.'">Select</a></td>';
	    $data['channels'] .= '<td><input type="text" class="text" id="on'.$x.'" maxlength="6" size="6" name="on'.$x.'" value="'.set_value('on'.$x, $row->views).'" /></td>';
        
        $data['channels'] .=  '</tr>';
        
        if($x == 1)
        {
           $data['colours'] = '#ch'.$x.'colour';
        }
        else
        {
           $data['colours'] .= ', #ch'.$x.'colour';
        }
        $x = $x + 1;
	  }
	  $this->load->view('home/settings/dev', $data);  
    }
    else
    {
      if(set_value('channel1') != '')
	  {
	  	  // Update normal channels
	  	  for($x=1;$x<=$channel->num_rows();$x++)
		  {
		  	  $main = 1;
			  $hide = 1;
			  if(set_value('main'.$x) == '')
			  {
			  	$main = 0;
			  }
			  
			  if(set_value('hide'.$x) == '')
			  {
			  	$hide = 0;
			  }
			  
			  if(set_value('on'.$x) == '')
			  {
			  	$views = 0;
			  }
			  else
			  {
			  	$views = set_value('on'.$x);
			  }
			  
	          $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
	                             'colour' => set_value('ch'.$x.'colour'),
	                             'main' => $main,
	                             'hide' => $hide,
	                             'views' => $views,
	                             'pulse' => 0);
								 
			  $this->channels_model->UpdateChannel(set_value('channel'.$x),$chUpdate);
		  }
		  
		  // Upate net channels
		  for($x=$channel->num_rows()+1;
		  	  $x<=($channel->num_rows()+$netCh->num_rows());
		  	  $x++)
		  {
			  $hide = 1;
			  
			  if(set_value('hide'.$x) == '')
			  {
			  	$hide = 0;
			  }
			  
			  $main = 1;
			  
			  if(set_value('main'.$x) == '')
			  {
			  	$main = 0;
			  }
			  
			 
			  if(set_value('on'.$x) == '')
			  {
			  	$views = 0;
			  }
			  else
			  {
			  	$views = set_value('on'.$x);
			  }
			  
	          $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
	                             'colour' => set_value('ch'.$x.'colour'),
	                             'views' => $views,
	                             'main' => $main,
	                             'hide' => $hide);
								 
			  $this->channels_model->UpdateChannel(set_value('channel'.$x),$chUpdate);
		  }
		  
		  // Update Pulse Channels
		  for($x=$channel->num_rows()+$netCh->num_rows()+1;
    	      $x<=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows();
    	      $x++)
	      {
	          $hide = 1;
			  
			  if(set_value('hide'.$x) == '')
			  {
			  	$hide = 0;
			  }
			  
			  
			  if(set_value('on'.$x) == '')
			  {
			  	$views = 0;
			  }
			  else
			  {
			  	$views = set_value('on'.$x);
			  }
			  
	          $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
	                             'colour' => set_value('ch'.$x.'colour'),
	                             'pulse' => set_value('pulse'.$x),
	                             'pulse_unit' => set_value('punit'.$x),
	                             'views' => $views,
	                             'hide' => $hide);
								 
			  $this->channels_model->UpdateChannel(set_value('channel'.$x),$chUpdate);
	      }
		
		  // Update temperature channels
	      for($x=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows()+1;
	      	  $x<=$channel->num_rows()+$netCh->num_rows()+$pulseCh->num_rows()+$tempCh->num_rows();
	    	  $x++)
	      {
	        $hide = 1;
			  
			  if(set_value('hide'.$x) == '')
			  {
			  	$hide = 0;
			  }
			  
			  
			  
			  if(set_value('on'.$x) == '')
			  {
			  	$views = 0;
			  }
			  else
			  {
			  	$views = set_value('on'.$x);
			  }

	          $chUpdate = array ('channel_name' => htmlspecialchars_decode(set_value('ch'.$x.'alias')),
	                             'colour' => set_value('ch'.$x.'colour'),
	                             'pulse_unit' => set_value('tmp'.$x.'Unit'),
	                             'views' => $views,
	                             'hide' => $hide);
								 
			  $this->channels_model->UpdateChannel(set_value('channel'.$x),$chUpdate);
	      }
		  
		  
		  // Insert or Remove Net channels
	      for($x=1;$x<$channel->num_rows()+1;$x++)
	      {
	          if(set_value('net'.$x) == 1)
			  {
			  	     if($this->channels_model->getChannelsNet(set_value('channel'.$x))->num_rows() == 0)
					 {
						 $this->channels_model->InsertNet(set_value('channel'.$x), set_value('device_id'), set_value('ch'.$x.'alias')." Gen", 0, $x, $this->_random_colour());
					 }   
			  }
			  else
			  {
			  		if($this->channels_model->getChannelsNet(set_value('channel'.$x))->num_rows() != 0)
					{
						 $this->channels_model->RemoveNet(set_value('channel'.$x));
					} 
			  }
	          
	     }
       }

       // Update device
       
	   if(set_value('net') == '')
	   {
		    $net = 0;
	   }
		
		
       $devUpdate = array( 'alias' => set_value('alias'),
                          'net' => $net,
                          'serial' => set_value('serial'));
						  
       $this->device_model->UpdateDevice($devUpdate,set_value('device_id'));
      
       redirect('/pages/device/geye/'.set_value('serial').'/1');
      }
    }
	
   // --------------------------------------------------------------------

   /**
	* Check for pulse values
	*
	* @access	private
	* @return	booolean
	*/
	private function pulseValue_check($str)
	{
		if (!is_numeric($str))
		{
			$this->form_validation->set_message('pulseValue_check', 'The %s field has to be numeric');
			return FALSE;
		}
		else
		{
			return TRUE;
		}
	}

   // --------------------------------------------------------------------

   /**
	* Random colour generator
	*
	* @access	private
	* @return	string
	*/
	private function _random_colour()
	{
	    mt_srand((double)microtime()*1000000);
	    $c = '';
	    while(strlen($c)<6){
	        $c .= sprintf("%02X", mt_rand(0, 255));
	    }
	    return $c;
	}
}

