<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * Load Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	Display
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class Load extends MX_Controller {
  
  /**
   * System settings
   *
   * @var object
   * @access private
   */
  private $sys;
  
  /**
   * Constructor
   *
   * @access	public
   */
  function Load()
  {
    parent::__construct();
    $this->load->helper('url');
    $this->load->helper('form');
    $this->load->model('channels_model');
    $this->load->model('sysinfo_model');
    $this->load->model('minute_model');
    $this->load->model('device_model');
    $this->load->model('hour_model');
    $this->load->model('virtual_model');
	
    $this->sys = $this->sysinfo_model->getInfo();
    $this->sys = $this->sys->row_array();
	
    date_default_timezone_set($this->sys["timezone"]);
	set_time_limit(1000); 
  }
 
   // --------------------------------------------------------------------

   /**
	* Get the channels information
	*
	* @access	public
    * @param	int
    * @param	int
    * @param	int
    * @param	int
    * @param	int
    * @param 	int
    * @param	int
	* @return	view
	*/
  function channelInfo($chan_id, $compare_id, $interval, $date, $gtype, $view, $showComp)
  {
  	// Get date information based on range selected
	$dateInfo = $this->_getDates($date,$interval,false);
	    	
    // Get main channel for totals
    $mainChan = $this->channels_model->getMain();
	
	$enableMain = 1;
	
	// If main doesn't exist, just get first channel
	if($mainChan->num_rows() == 0)
	{
		$enableMain = 0;
		if($compare_id == 0)
		{
			$fChan = $this->channels_model->getFirstChannel();
			$fChan = $fChan->row_array();
			$compare_id = $fChan['channel_id'];
		}
	}
	
	// Get power usage between two dates
    $result = $this->_getChannelPower($date, $dateInfo['begin'], $dateInfo['end'], $chan_id, 0);
	$data['usedTkWh'] = round($result['usedTkWh'],2);
	
	// If not main, compare usage to mains
	if(isset($data['mainPower']))
	{
		if($data['mainPower'] != 0)
		{
			$data['percOfMain'] = round(($result['usedTkWh']/$data['mainPower'])*100);
		}
		else
		{
			$data['percOfMain'] = 0;
		}
		
		if($data['percOfMain'] > 100)
		{
			$data['percOfMain'] = 100;
		}
	}
	else {
		$data['mainPower'] = 0;
	}
	
	// Vars for display
    $data['usedTCost'] = "$".sprintf("%01.2f", round($result['usedTCost'],2));
    $data['avgWatt'] = round($result['avgWatt']);
	$data['maxWatt'] = round($result['maxWatt']);
	$data['minWatt'] = round($result['minWatt']);
	
	
	// Do same thing for comparsion channel
	$result = $this->_getChannelPower($date, $dateInfo['begin'], $dateInfo['end'], $compare_id, 1);
    $data['compareTkWh'] = round($result['usedTkWh'],2);
	if($data['mainPower'] != 0)
	{
		$data['compPercOfMain'] = round(($result['usedTkWh']/$data['mainPower'])*100);
	}
	else
	{
		$data['compPercOfMain'] = 0;
	}
	
	if($data['compPercOfMain'] > 100)
	{
		$data['compPercOfMain'] = 100;
	}
	
    $data['compareTCost'] = "$".sprintf("%01.2f", round($result['usedTCost'],2));
    $data['compAvgWatt'] = round($result['avgWatt']);
	$data['compMaxWatt'] = round($result['maxWatt']);
	$data['compMinWatt'] = round($result['minWatt']);
	
	
	// Get information on the channel
	$lastRead = $this->minute_model->getLastReading($chan_id, $dateInfo['begin']);
	$chinfo = $this->channels_model->getChannelInfo($chan_id);
	$chinfo = $chinfo->row_array();
	$data['chName'] = $chinfo['channel_name'];
	$data['chImage'] = $chinfo['image'];
	$data['chUnit'] = $chinfo['pulse_unit'];

    // If running, get last reading
    if($lastRead->num_rows() > 0)
    {
      $avgWatts = $lastRead->row_array();
      $data['lastReading'] = date("F j, g:i a", strtotime($avgWatts['date'])+$dateInfo['offset']);
	  if($chinfo['ctype'] != 1)
	  {
	  	 $data['lastWatt'] = $avgWatts['watt'];
	  }
	  else {
	  	 $data['lastWatt'] = $avgWatts['kwh'];
	  }
    }
	
	
	// If not running, last reading is null
    if($lastRead->num_rows() == 0)
    {
      $data['lastReading'] = "---";
	  $data['lastWatt'] = 0;
    }
    
	
	
    $data['compLastReading'] = 0;
    $data['compLastWatt'] = 0;
    
	// If no comparison selecte, default to sum of main channels
    if($compare_id == 0)
    {
      $mainChan = $this->channels_model->getMain();
	  
	  $cWatt = -1;
	  $first = TRUE;
	  $powerInfo = '';
	  foreach($mainChan->result() as $mainCh)
      {
      	$powInfo = $this->minute_model->getLastReading($mainCh->channel_id, $dateInfo['begin']);
	    if($powInfo->num_rows() > 0)
	    {
	    	if($first)
			{
				$cWatt++;
				$first = FALSE;
			}
			
        	$powerInfo = $powInfo->row_array();
			$cWatt += $powerInfo['watt'];
		}
	  }
	  
      $compInfo['ctype'] = 0;
      
	  //  Make sure there's a reading, if not then it's null
      if($cWatt != -1)
      {
        $data['compLastReading'] = date("F j, g:i a", strtotime($powerInfo['date'])+$dateInfo['offset']);
        $data['compName'] = 'Sum of Mains';
    	$data['compLastWatt'] = $cWatt;
		
      }
      else
      {
        $data['compLastReading'] = "---";
        $data['compName'] = 'Sum of Mains';
      }
    }
    else  // If channel is selected, get its readings
	{
      
      $compLast = $this->minute_model->getLastReading($compare_id, $dateInfo['begin']);
      $compInfo = $this->channels_model->getChannelInfo($compare_id);
      $compInfo = $compInfo->row_array();
      $data['compName'] = $compInfo['channel_name'];
	  $data['compImg'] = $compInfo['image'];
	  $data['compUnit'] = $compInfo['pulse_unit'];

      if($compLast->num_rows() > 0)
      {
        $compLast = $compLast->row_array();
        $data['compLastReading'] = date("F j, g:i a", strtotime($compLast['date'])+$dateInfo['offset']);
		if($compInfo['ctype'] != 1)
		{
    		$data['compLastWatt'] = $compLast['watt'];
		}
		else {
		  	$data['compLastWatt'] = $compLast['kwh'];
		}
      }
      else
      {
        $data['compLastReading'] = "---";
      }
    }

    // Add Sum of Mains option to comparison channel
    $data['html'] = '';
    $data['chanList'] = '';
	if($enableMain == 1)
	{     
		$data['html'] .= '<option ';
	    
	    if($compare_id == 0)
	    {
	      $data['html'] .= 'selected="true"';
	    }
	    
	    $data['html'] .= ' value="'.base_url().'index.php/pages/load/channelInfo/'.$chan_id.'/0/'.$interval.'/'.$date.'/'.$gtype.'/1/0/1">Sum of Mains</option>';
	}
	
    $channels = $this->channels_model->channelSearch("");
	
	// Get virtual channels
	$virtDev = $this->device_model->getDevice('9999999');
	$compIn = '';
	$chanIn = '';
			
	if($virtDev->num_rows() > 0)
	{
		$virtDev = $virtDev->row_array();
		$virt = $this->channels_model->getVirtChannels($virtDev['device_id']);
		
		// Make list of virt channels, append them onto the start of the channel lists
		$data['html'] .= '<optgroup label="Virtual Channels">';
		$data['chanList'] .= '<optgroup label="Virtual Channels">';
	    foreach($virt->result() as $virtual)
	    {
	      $data['html'] .= '<option ';
		  $data['chanList']	.= '<option ';  
		  
		  // If the comparsion channel is a virtual channel, add a list of channels 
		  // that virtual channel is comprised of to the opposing list.
		  if($compare_id == $virtual->channel_id) 
		  {
		  	$data['html'] .= 'selected="selected"';
			$compIn .= '<optgroup label="Channels in '.$data['compName'].'">';
			$inVirt = $this->virtual_model->GetChannelsModule($virtual->channel_id);
			foreach($inVirt->result() as $virtChan)
			{
				$compIn .= '<option value="'.base_url().'index.php/pages/load/channelInfo/'.$virtChan->channel_id.'/'.$compare_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$virtChan->channel_name.'</option>';
			}
			$compIn .= '</optgroup>';
		  }
		  
		  // If the channel is a virtual channel, add a list of channels 
		  // that virtual channel is comprised of to the opposing list.
		  if($chan_id == $virtual->channel_id)
		  {
		  	$data['chanList'] .= 'selected="selected"';
			$chanIn .= '<optgroup label="Channels in '.$data['chName'].'">';
			$inVirt = $this->virtual_model->GetChannelsModule($virtual->channel_id);
			
			foreach($inVirt->result() as $virtChan)
			{
				$chanIn .= '<option value="'.base_url().'index.php/pages/load/channelInfo/'.$chan_id.'/'.$virtChan->channel_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$virtChan->channel_name.'</option>';
			}
			$chanIn .= '</optgroup>';
		  }
		  $data['html'] .= ' value="'.base_url().'index.php/pages/load/channelInfo/'.$chan_id.'/'.$virtual->channel_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$virtual->channel_name.'</option>';
		  $data['chanList'] .= ' value="'.base_url().'index.php/pages/load/channelInfo/'.$virtual->channel_id.'/'.$compare_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$virtual->channel_name.'</option>';
	    }
		$data['chanList'] .= '</optgroup>';
	    $data['html'] .= '</optgroup>';
		
		// If comprising channels exist, append them
		if(strlen($compIn) > 0)
		{
			$data['chanList'] .= $compIn;
		}
		
		if(strlen($chanIn) > 0)
		{
			$data['html'] .= $chanIn;
		}
		
	}
	
	
	// Get the list of all channels
	$data['html'] .= '<optgroup label="All Channels">';
	$data['chanList'] .= '<optgroup label="All Channels">';
    foreach($channels->result() as $channel)
    {
      $data['html'] .= '<option ';
	  $data['chanList']	.= '<option ';  
	  if($compare_id == $channel->channel_id)
	  {
	  	$data['html'] .= 'selected="selected"';
	  }
	  
	  if($chan_id == $channel->channel_id)
	  {
	  	$data['chanList'] .= 'selected="selected"'; 
	  }
	  $data['html'] .= ' value="'.base_url().'index.php/pages/load/channelInfo/'.$chan_id.'/'.$channel->channel_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$channel->channel_name.'</option>';
	  $data['chanList'] .= ' value="'.base_url().'index.php/pages/load/channelInfo/'.$channel->channel_id.'/'.$compare_id.'/'.$interval.'/'.$date.'/'.$gtype.'/1/1">'.$channel->channel_name.'</option>';
    }
	$data['chanList'] .= '</optgroup>';
    $data['html'] .= '</optgroup>';
	
	
    $data['html'] = '<select class="text" onchange="location.href=this.options[this.selectedIndex].value" style="width:200px;">'.$data['html'].'</select>';
    $data['chanList'] = '<select class="text" onchange="location.href=this.options[this.selectedIndex].value" style="width:200px;">'.$data['chanList'].'</select>';
    
	// Check if channel can possibly be negative, used for graph scales
	$data['netTemp'] = false;
	
	if($this->channels_model->checkNegatives($chan_id)->num_rows() > 0 || $this->channels_model->checkNegatives($compare_id)->num_rows() > 0)
	{
		$data['netTemp'] = true;	
	}
	
	// Grab the values that get passed through the functions
    $data['gtype'] = $gtype;
    $data['interval'] = $interval;
    $data['date'] = $date;    
    $data['chid'] = $chan_id;
    $data['compid'] = $compare_id;
    $data['titleBegin'] = $dateInfo['titleBegin'];
	$data['today'] = $dateInfo['today'];
	$data['ctype'] = $chinfo['ctype'];
	$data['comptype'] = $compInfo['ctype'];
	$data['theDate'] = $dateInfo['theDate'];
	$data['showComp'] = $showComp;
	
    $this->load->view('home/loads/load', $data);
  }
  
   // --------------------------------------------------------------------

   /**
	* Get Load Info for the Graph
	*
	* @access	public
    * @param	int
    * @param	int
    * @param	int
    * @param	int
    * @param	int
	* @return	JSON
	*/
  function GetLoadInfo($chan_id,$compare_id,$interval,$date,$gtype)
  {
  	// Get the date intervals
	$dateInfo = $this->_getDates($date,$interval);
	
	$data = "";
	$first = TRUE;
	$temp = FALSE;
	$pulse = FALSE;
	$watt = FALSE;
	$kwh = FALSE;
	$type = "";
	$side = "";
	$coName = '';
	$chWatt = FALSE;
	$coWatt = FALSE;
	$labels = '';
	$left = FALSE;
	
    // Determine channel type and define the axis and units accordingly
	$chinfo = $this->channels_model->getChannelInfo($chan_id);
	$chinfo = $chinfo->row_array();
	  if(!$first)
      {
	    $data .= ", ";
	    $side .= ", ";
	    $type .= ", ";
      }
      else {
	    $first = FALSE;
  	  }
	  
    $data .= '"'.$chinfo['channel_name'].'_col": "#'.$chinfo['colour'].'"';
	
  	if($chinfo['ctype'] == 1)  // Temperature
	{
		$temp = TRUE;
		$side .= '"'.$chinfo['channel_name'].'_side": "valueAxis"';
		$type .= '"'.$chinfo['channel_name'].'_val": "'.$chinfo['pulse_unit'].'"';
		$data .= ', "kaxis": "'.$chinfo['pulse_unit'].'"';
	}
	else if($chinfo['ctype'] == 2) // Pulse
	{
		$pulse = TRUE;
		$type .= '"'.$chinfo['channel_name'].'_val": "'.$chinfo['pulse_unit'].'"';
		$side .= '"'.$chinfo['channel_name'].'_side": "valueAxis"';
		$data .= ', "kaxis": "'.$chinfo['pulse_unit'];
		
		if($date == 0 && $gtype == 1)
		{
			$data .= ' (per min.)"';
		}
		else
		{
			$data .= '"';
		}
	}
	else if($chinfo['ctype'] == 3)  // Volts
	{
		$volt = TRUE;
		$type .= '"'.$chinfo['channel_name'].'_val": "Volts"';
		$side .= '"'.$chinfo['channel_name'].'_side": "valueAxis"';
		$data .= ', "kaxis": "Volts"';
	}
	else if($date == 0 && ($gtype == 1|| $gtype == 2))  // Watts
	{
		$watt = TRUE;
		$chWatt = TRUE;
		$type .= '"'.$chinfo['channel_name'].'_val": "Watts"';
		$side .= '"'.$chinfo['channel_name'].'_side": "valueAxis"';
		$data .= ', "kaxis": "Watts"';
	}
	else  // Kwh
	{
		$kwh = TRUE;
		$type .= '"'.$chinfo['channel_name'].'_val": "kWh"';
		$side .= '"'.$chinfo['channel_name'].'_side": "valueAxis"';
		$data .= ', "kaxis": "kWh"';
	}
	$chName = $chinfo['channel_name'];
	
	
	if($gtype != 2)
	{
	    if($compare_id == 0 && $compare_id != $chan_id)
	    { 
		  if(!$first)
	      {
		    $data .= ", ";
		    $side .= ", ";
		    $type .= ", ";
	      }
	      else {
		    $first = FALSE;
	  	  }
		  
		  if($date == 0 && ($gtype == 1 || $gtype == 2)){
			  $data .= '"Sum of Mains_col": "#FF0000"';
			  $type .= '"Sum of Mains_val": "Watts"';
			  if($watt)
			  {
			  	$side .= '"Sum of Mains_side": "valueAxis"';	
			  }
			  else {
				  $side .= '"Sum of Mains_side": "valueAxis3"';
			  	  $data .= ', "paxis": "Watts"';
			  }
			  
			  $watt = TRUE;
			  $coWatt = TRUE;
			  $coName = "Sum of Mains";
		  }
		  else
		  {	
		  	  $data .= '"Sum of Mains_col": "#FF0000"';
			  $type .= '"Sum of Mains_val": "kWh"';
			  if($kwh)
			  {
			  	$side .= '"Sum of Mains_side": "valueAxis"';	
			  }
			  else {
				  $side .= '"Sum of Mains_side": "valueAxis3"';
			  	  $data .= ', "paxis": "kWh"';
			  }
			  $kwh = TRUE;
			  $coName = "Sum of Mains";
		  }
	    }
	    else if ($compare_id != $chan_id)
	    {
      
	      $compInfo = $this->channels_model->getChannelInfo($compare_id);
	      $compInfo = $compInfo->row_array();
		  if(!$first)
	      {
		    $data .= ", ";
		    $side .= ", ";
		    $type .= ", ";
	      }
	      else {
		    $first = FALSE;
	  	  }
		  
		  $data .= '"'.$compInfo['channel_name'].'_col": "#'.$compInfo['colour'].'"';
		  
	      	if($compInfo['ctype'] == 1)
			{
				if($temp)
				{
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis"';	
				}
				else {
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis3"';
				}
				
				$type .= '"'.$compInfo['channel_name'].'_val": "'.$compInfo['pulse_unit'].'"';
			    $data .= ', "paxis": "'.$compInfo['pulse_unit'].'"';
			}
			else if($compInfo['ctype'] == 2)
			{
				$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis3"';	
				
				$type .= '"'.$compInfo['channel_name'].'_val": "'.$compInfo['pulse_unit'].'"';
				$data .= ', "paxis": "'.$compInfo['pulse_unit'].'"';
			}
			else if($compInfo['ctype'] == 3)
			{
				if($volt)
				{
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis"';	
				}
				else {
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis3"';
				}
				
				$type .= '"'.$compInfo['channel_name'].'_val": "Volts"';
				$data .= ', "paxis": "Volts"';
			}
			else if($date == 0 && ($gtype == 1|| $gtype == 2)){
				
				$type .= '"'.$compInfo['channel_name'].'_val": "Watts"';
			    if($watt)
			    {
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis"';
			    }
			    else {
				  $side .= '"'.$compInfo['channel_name'].'_side": "valueAxis3"';
			  	  $data .= ', "paxis": "Watts"';
			    }
			}
			else
			{
				$kwh = TRUE;
				$type .= '"'.$compInfo['channel_name'].'_val": "kWh"';
			    if($kwh)
			    {
					$side .= '"'.$compInfo['channel_name'].'_side": "valueAxis"';
			    }
			    else {
				  $side .= '"'.$compInfo['channel_name'].'_side": "valueAxis3"';
			  	  $data .= ', "paxis": "kWh"';
			    }
			}
			
		    $coName = $compInfo['channel_name'];
		  
	  
	    }
	}
	
	
	// Get average power for comparison and channel
  	$compWatt = $this->_getChannelPower($date, $dateInfo['begin'], $dateInfo['end'], $compare_id, 1);
    $data .= ', "'.$coName.'_avg"'.':'.round($compWatt['avgWatt']);
	
  	$cWatt = $this->_getChannelPower($date, $dateInfo['begin'], $dateInfo['end'], $chan_id, 0);
    $data .= ', "'.$chName.'_avg"'.':'.round($cWatt['avgWatt']);
	$maxval = 0;
	$minWatt = 0;
	
	// If looking at watts, set a scale if values are low so they aren't misleading
	if($watt)
	{
		if($coWatt && $chWatt)
		{
			if($compWatt['avgWatt'] < 100 && $cWatt['avgWatt'] < 100)
			{
				$maxval = 1000;
			}
		}
		else if($chWatt)
		{
			if($cWatt['avgWatt'] < 100)
			{
				$maxval = 1000;
			}
		}
		else if($coWatt)
		{
			if($compWatt['avgWatt'] < 100)
			{
				$maxval = 1000;
			}
		}
				
		if($compWatt['minWatt'] <= $cWatt['minWatt'])
		{
			$minWatt = $compWatt['minWatt']*5;
		}
		else {
			$minWatt = $cWatt['minWatt']*5;
		}
	}	
	
	
	header("Content-type: text/javascript");
	echo '[{'.$data.','.$type.','.$side.',"chnames":"'.$chName;
	
	if($coName != '')
	{
		echo '<n>'.$coName;
	}
	
	echo '","maxval":"'.$maxval.'", "minWatt":"'.$minWatt.'", "title":"'.$dateInfo['titleBegin'].'"}]';
  }

   // --------------------------------------------------------------------

   /**
	* Generate JSON for Line Graph
	*
	* @access	public
    * @param	int
    * @param	int
    * @param	int
    * @param	int
	* @return	JSON
	*/
  function loadLineGraph($chan_id,$compare_id,$interval,$date)
  {
  	// Get date range for Line Graph
   	$dateInfo = $this->_getDates($date,$interval);
   
	$chinfo = $this->channels_model->getChannelInfo($chan_id);
	$chinfo = $chinfo->row_array();

	$chName = $chinfo['channel_name'];
	
	
    if($compare_id == 0)
    {
		$coName = "Sum of Mains";
		$compInfo['ctype'] = 0;
    }
    else
    {
  	  $compInfo = $this->channels_model->getChannelInfo($compare_id);
      $compInfo = $compInfo->row_array();
	  $coName = $compInfo['channel_name'];
    }
	
    if($date == 0)
    {
      
  	  $info = $this->minute_model->graphLoad($chan_id, $dateInfo['begin'], $dateInfo['end']);	  
	  
      if($compare_id == 0)
      {
        $comp = $this->minute_model->graphMain($dateInfo['begin'], $dateInfo['end']);
      }
      else
      {
    	$comp = $this->minute_model->graphLoad($compare_id, $dateInfo['begin'], $dateInfo['end']);
      }
      
      $num = 60;
	  $type = 0;
    }
    else
    {
      $info = $this->hour_model->graphLoad($chan_id, $dateInfo['begin'], $dateInfo['end'], $dateInfo['graphForm']);
	  
      
      if($compare_id == 0)
      {
        $comp = $this->hour_model->graphMain($dateInfo['begin'], $dateInfo['end'],$dateInfo['graphForm']);
      }
      else
      {
    	$comp = $this->hour_model->graphLoad($compare_id, $dateInfo['begin'], $dateInfo['end'], $dateInfo['graphForm']);
      }
	  $type = 1;
    }
	if($chName == $coName)
	{
		$coName .= "*";
	}
	$tmpVal = 0;
	$kwh = 0;
	$y = 0;
	
	date_default_timezone_set('UTC');
	while ($v = $info->_fetch_object())
    {	  
	  if($chinfo['ctype'] == 1 || $chinfo['ctype'] == 3) 
	  	$tmpVal = round($v->kwh,3);		  
	  else if($date > 0)
		$tmpVal += round($v->kwh,3);
	  	  
	  $index = round((strtotime($v->date)-$dateInfo['genIndex'])/$dateInfo['indVal'])-1;
	  
	  if($date == 0 && $chinfo['ctype'] == 0)
	  {
	  	  $tmpVal = $v->watt;
		  
		  if(!isset($data[$index]))
		  	$data[$index] = ' "'.$chName.'": '.$tmpVal;
		  else
		    $data[$index] .= ', "'.$chName.'": '.$tmpVal;
		  
	  }
	  else {
	  	  if(!isset($data[$index]))
		  	$data[$index] = ' "'.$chName.'": '.round($tmpVal,3);
		  else
		    $data[$index] .= ', "'.$chName.'": '.round($tmpVal,3);
	  }
    }

	$info->free_result();
    $tmpVal = 0;	
	$kwh = 0;	
    while ($v = $comp->_fetch_object()) 
    {
      if($compInfo['ctype'] == 1 || $compInfo['ctype'] == 3) 
      {
	  	$tmpVal = round($v->kwh,3);	
	  }
      else if($date > 0)
	  {
	  	$tmpVal += round($v->kwh,3);
	  }
	  
	  $index = round(((strtotime($v->date)-$dateInfo['genIndex'])/$dateInfo['indVal']))-1;
	  
	  if($date == 0 && $compInfo['ctype'] == 0)
	  {
	  	  $tmpVal = $v->watt;
		  
		  if(!isset($data[$index]))
		  {
			 $data[$index] = ' "'.$coName.'": '.$tmpVal;
		  }
		  else
		  {
		     $data[$index] .= ', "'.$coName.'": '.$tmpVal;
		  }
	  }
	  else {
		  if(!isset($data[$index]))
		  {
			 $data[$index] = ' "'.$coName.'": '.round($tmpVal,3);
		  }
		  else
		  {
		     $data[$index] .= ', "'.$coName.'": '.round($tmpVal,3);
		  }
	  }
    }
	date_default_timezone_set($this->sys['timezone']);
	
	$comp->free_result();
    
	$first = TRUE;
	$x = 0;
	echo '[';
	foreach($dateInfo['dateArray'] as $value)
    {
      if(!$first)
	  {
	  	echo ', ';
	  }
	  else {
		  $first = FALSE;
	  }
	  
	  if(isset($data[$x]))
	  {
      	echo '{ "date": "'.date($dateInfo['format'],$value).'", '.$data[$x].' }';
	  }
	  else
	  {
	  	echo '{ "date": "'.date($dateInfo['format'],$value).'" }';
	  }
	  $x++;
    }
	echo ']';
  }

   // --------------------------------------------------------------------

   /**
	* Generate JSON for Bar Graph
	*
	* @access	public
    * @param	int
    * @param	int
    * @param	int
    * @param	int
	* @return	JSON
	*/
  //function loadBarGraph($chan_id,$compare_id,$interval,$date)
  function loadBarGraph($chan_id,$compare_id,$interval,$date)
  {	
    if($date == 0)
    {
      $dateInfo['start'] = mktime(1, 0, 0, date("m") , date("d")-$interval, date("Y"));
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(0, 59, 59, date("m") , date("d")+1-$interval, date("Y")));
      $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
	  $userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
	  
      $dateInfo['format'] = 'h:00 A';
	  $dateInfo['graphForm'] = 'YYYY-MM-DD HH24:00:00';
	  $dateInfo['strFormat'] = 'Y-m-d H:00:00';
	  $dateInfo['indVal'] = 3600;
	  $dateInfo['genIndex'] = $dateInfo['start']+1-3600;
      $dateInfo['dateArray'] = $this->_getHours($dateInfo['begin'],$dateInfo['end']);
    } 
	elseif($date == 1)
    {
      $dateInfo['start'] = mktime(0, 0, 0, date('m'), date('d')-date('w')-($interval*7)+1, date('Y'));
      $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
      $userTimezone = new DateTime(date('Y-m-d H:i:s', $dateInfo['start']), new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
	
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', $dateInfo['start'] + (7 * 24*60*60));
	  
	  $dateInfo['graphForm'] = 'YYYY-MM-DD HH24:00:00';
	  $dateInfo['strFormat'] =  'Y-m-d H:00:00';
      $dateInfo['format'] = 'M d H:00';
	  $dateInfo['indVal'] = 3600;
	  $dateInfo['genIndex'] = $dateInfo['start']+1-3600;
      $dateInfo['dateArray'] = $this->_getHours($dateInfo['begin'],$dateInfo['end'],$dateInfo['offset']);
    }
    elseif($date == 2)
    {
      $dateInfo['start'] = mktime(00, 00, 01, date("m")-$interval, $this->sys['month_start'], date("Y"));
      $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
      $userTimezone = new DateTime(date('Y-m-d H:i:s', $dateInfo['start']), new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
	
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, date("m")-$interval+1, $this->sys['month_start']-1, date("Y")));
	  
	  $dateInfo['graphForm'] =  'YYYY-MM-DD';
	  $dateInfo['strFormat'] =  'Y-m-d';
      $dateInfo['format'] = 'M d';
	  $dateInfo['indVal'] = 86400;
	  $dateInfo['genIndex'] = $dateInfo['start']+1-86400;
      $dateInfo['dateArray'] = $this->_getDays($dateInfo['begin'],$dateInfo['end'],$dateInfo['offset']);
    }
    else
    {
      $dateInfo['start'] = mktime(23, 59, 59, 1, 1, date("Y")-$interval);
      $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
      $userTimezone = new DateTime(date('Y-m-d H:i:s', $dateInfo['start']), new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
	  
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, 1, $this->sys['month_start']-1, date("Y")-($interval-1)));
	  
	  $dateInfo['endDate'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, 12, 31, date("Y")-$interval));
	  
	  $dateInfo['graphForm'] =  'YYYY-MM';
	  $dateInfo['strFormat'] =  'Y-m';
      $dateInfo['format'] = 'M Y';
	  $dateInfo['indVal'] = 2628000;
	  $dateInfo['genIndex'] = $dateInfo['start']+1-2628000;
      $dateInfo['dateArray'] = $this->_getMonths($dateInfo['begin'],$dateInfo['endDate'],$dateInfo['offset']);
    }
    
	$chinfo = $this->channels_model->getChannelInfo($chan_id);
	$chinfo = $chinfo->row_array();
	$chName = $chinfo['channel_name'];
	$ctype = $chinfo['ctype'];

	
    if($compare_id == 0)
    {
		$coName = "Sum of Mains";
		$compType = 0;
    }
    else
    {
      $compInfo = $this->channels_model->getChannelInfo($compare_id);
      $compInfo = $compInfo->row_array();
	  $coName = $compInfo['channel_name'];
	  $compType = $compInfo['ctype'];
    }

    $info = $this->hour_model->graphLoad($chan_id, $dateInfo['begin'], $dateInfo['end'], $dateInfo['graphForm'], $ctype);
	
    if($compare_id == 0)
    {
        $comp = $this->hour_model->graphMain($dateInfo['begin'], $dateInfo['end'],$dateInfo['graphForm']);
	}
    else
    {
       	$comp = $this->hour_model->graphLoad($compare_id, $dateInfo['begin'], $dateInfo['end'], $dateInfo['graphForm'], $compType);
    }
	$tmpVal = 0;
	$data = '';
	$compData = '';
	$x = 0;
	$infoKwh = array();
	$compKwh = array();
	$total = 0;
	$tempIndex = 0;
	if($date > 1)
	{
		  date_default_timezone_set('UTC');
		  $dt = new DateTime(null, new DateTimeZone($this->sys['timezone']));
	}
    if($chName == $coName)
	{
		$coName .= "*";
	}

	$tempDate = '';
	$indHold = 0;
	$recCnt = 1;
	$y = 0;
	$modifier = '-'.($this->sys['month_start']-1).' days';
	$dtz = new DateTimeZone($this->sys['timezone']);
	while ($result = $info->_fetch_object())
    {
	  if($date < 2)
	  {
		  $index = strtotime($result->date)-$dateInfo['genIndex'];
	
	      $userTimezone = new DateTime(date('Y-m-d H:i:s', strtotime($result->date)), new DateTimeZone($this->sys['timezone']));
	  	  $dateInfo['offset'] = $userTimezone->getOffset();
		  $index += $dateInfo['offset'];
		  $index = floor($index/$dateInfo['indVal']);
	  }
	  else if($date == 2)
	  {
		  $dt->setTimestamp(strtotime($result->date));
		  $index = $dt->format('d')-$this->sys['month_start'];
		  
		  if($index >= 0)
		  {
		  	$tempIndex = $index;
		  }
		  else
		  {
		  	$index = $index+$this->sys['month_start']+$tempIndex;	
		  }
	  }
	  else {
		$dt = DateTime::createFromFormat('Y-m-d H:i:s', $result->date);
		$dt->modify($modifier);
		$dt->setTimezone($dtz);
		$index = $dt->format('m')-1;
		unset($dt);
	  }
	  
	  if($ctype == 3 && $date > 1)
	  {
		  if($index == $indHold)
		  {
		  	$recCnt++;
		  }
		  else
		  {
		  	if(isset($infoKwh[$indHold]))
			{
		  		$infoKwh[$indHold] = $infoKwh[$indHold]/$recCnt;
			}
			
		  	$indHold = $index;
			$recCnt = 1;
		  }
	  }
	  
	  if(!isset($infoKwh[$index]))
	  {
	  	$infoKwh[$index] = 0;
	  }
	  
	  if(!isset($data[$index]))
	  {
		 $data[$index] = ' "'.$chName.'": ';
	  }
	  
	  $tmpVal = round($result->kwh,3);	  		
	  
  	  if($ctype != 1)
	  {
	  	$infoKwh[$index] += $tmpVal;
	  }
	  else
	  {
	  	$infoKwh[$index] = $tmpVal;
	  }

	  $x++;
    }

	if(isset($infoKwh[$indHold]))
	{
  		$infoKwh[$indHold] = $infoKwh[$indHold]/$recCnt;
	}

	$tempIndex = 0;


	$indHold = 0;
	$recCnt = 1;
	
    while ($result = $comp->_fetch_object())
    {
	  if($date < 2)
	  {
		  $index = strtotime($result->date)-$dateInfo['genIndex'];
	
	      $userTimezone = new DateTime(date('Y-m-d H:i:s', strtotime($result->date)), new DateTimeZone($this->sys['timezone']));
	  	  $dateInfo['offset'] = $userTimezone->getOffset();
		  $index += $dateInfo['offset'];
		  $index = floor($index/$dateInfo['indVal']);
	  }
	  else if($date == 2)
	  {
		  $dt->setTimestamp(strtotime($result->date));
		  $index = $dt->format('d')-$this->sys['month_start'];
		  		  
		  if($index >= 0)
		  {
		  	$tempIndex = $index;
		  }
		  else
		  {
		  	$index = $index+$this->sys['month_start']+$tempIndex;	
		  }
	  }
	  else {
	  		$dt = DateTime::createFromFormat('Y-m-d H:i:s', $result->date);
		 	$dt->modify($modifier);
			$dt->setTimezone($dtz);
		 	$index = $dt->format('m')-1;
			unset($dt);
	  }

	  if($compType == 3 && $date > 1)
	  {
		  if($index == $indHold)
		  {
		  	$recCnt++;
		  }
		  else
		  {
		  	if(isset($compKwh[$indHold]))
			{
		  		$compKwh[$indHold] = $compKwh[$indHold]/$recCnt;
			}
			
		  	$indHold = $index;
			$recCnt = 1;
		  }
	  }
	  
	  if(!isset($compKwh[$index]))
	  {
	  	$compKwh[$index] = 0;
	  }
	  
	  if(!isset($compData[$index]))
	  {
		 $compData[$index] = ' "'.$coName.'": ';
	  }
	  
	  $tmpVal = round($result->kwh,3);	  		
	  	  
  	  if($compType != 1)
	  {
	  	$compKwh[$index] += $tmpVal;
	  }
	  else
	  {
	  	$compKwh[$index] = $tmpVal;
	  }
    }
	$firstRec = TRUE;
	$firstNull = FALSE;
	$x = 0;

  	if(isset($compKwh[$indHold]))
	{
  		$compKwh[$indHold] = $compKwh[$indHold]/$recCnt;
	}

	$chnKwh = 0;
	$sndKwh = 0;
	$chUpd = FALSE;
	$sndUpd = FALSE;
	$compUpd = FALSE;
	$cEnd = FALSE;
	$compEnd = FALSE;
	
	foreach($dateInfo['dateArray'] as $value)
    {
      if(isset($data[$x]))
	  {
	    if($chUpd)
		{
			$infoKwh[$x] = $chnKwh;
			$chUpd = FALSE;
		}
		else if ($chnKwh > 0 && $date == 2)
		{
			$infoKwh[$x] = $infoKwh[$x]-$chnKwh;
			$chnKwh = 0;
		}
	    else if($chnKwh > 0)
		{
			$infoKwh[$x] = $chnKwh;
			$chnKwh = 0;
		}
	  }
	  else if (count($infoKwh) > 0 && $chan_id != 0) {
	  	$fMat = "";
		/*if($date == 2)
		{
			$fMat = 'YYYY-MM-DD';
		}*/
		$last = $this->hour_model->getLastHour($chan_id, date('Y-m-d H:i:s',$value-($dateInfo['offset'])), $fMat);
		$first = $this->hour_model->getFirstHour($chan_id, date('Y-m-d H:i:s',$value-($dateInfo['offset'])), $fMat);		
		
		if($date < 2 && $first->num_rows() == 0 && !$cEnd)
		{
			$totKwh = $this->minute_model->getSumForward($chan_id,  date('Y-m-d H:i:s',$value-3600-($dateInfo['offset'])));
			
			if($totKwh->num_rows() > 0)
			{
				$totKwh = $totKwh->row_array();
				$chUpd = TRUE;
				if($ctype != 1 && $ctype != 3)
				{
					$chnKwh = $totKwh['kwh'];
				}
				else
				{
					$chnKwh = round($totKwh['tmp'],3);
				}
				$data[$x] =  ' "'.$chName.'": ';
				$infoKwh[$x] = $chnKwh;
			}
			$cEnd = TRUE; 
		}
		
		if($first->num_rows() > 0 && $last->num_rows() > 0 && !$cEnd)
		{
			if($first->num_rows() > 1)
			{
				$testOne = $first->row_array(0);
				$testTwo = $first->row_array(1);
				
				if((strtotime($testTwo['date'])-strtotime($testOne['date'])) < 3601 && $date < 2)
				{
					$first = $testTwo;
				}
				else if((strtotime($testTwo['date'])-strtotime($testOne['date'])) < 86401 && $date == 2)
				{
					$first = $testOne;
				}
				else {
					$first = $testOne;
				}
			}
			else {
				$first = $first->row_array();
			}
			$last = $last->row_array();

			if($date < 2)
			{
				$hours = sizeof($this->_getHours(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']));
			}
			else if($date == 2)
			{
				$hours = sizeof($this->_getDays(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']))-1;
			}
			else
			{
				$hours = sizeof($this->_getMonths(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']))-1;	
			}
			if($date != 2)
			{
				if(($last['seconds']-$first['seconds'])!=0)
				{
					if($last['seconds'] > $first['seconds'])
					{
				        $sec = pow(256,3) - $last['seconds'];
				        $sec += $first['seconds'];
					}
					else
					{
						$sec = ($first['seconds']-$last['seconds']);
					}
					
					if($last['wattsecond'] > $first['wattsecond'])
			        {
				          $wsec = pow(256,5) - $last['wattsecond'];
				          $wsec += $first['wattsecond'];
						  $kWh = $wsec/3600000;
			        }
			        else
			        {
			          	 $wsec = $first['wattsecond'] - $last['wattsecond'];
						 $kWh = $wsec/3600000;
			        } 
	
					if($kWh > 20000)
					{
						$kWh = 0;
					}
	
					$chUpd = TRUE;
					$chnKwh = round($kWh/$hours,3);
					$data[$x] =  ' "'.$chName.'": ';
					$infoKwh[$x] = $chnKwh; 
				}
				
			}
			else
			{
				$chnKwh = $first['kwh']/$hours;	
				$data[$x] =  ' "'.$chName.'": ';
				$infoKwh[$x] = $chnKwh; 
			}
	  	 }
	  }

	  if(isset($compData[$x]))
	  {
	    if($sndUpd)
		{
			$compKwh[$x] = $sndKwh;
			$sndUpd = FALSE;
		}
		else if ($sndKwh > 0 && $date == 2)
		{
			$compKwh[$x] = $compKwh[$x]-$sndKwh;
			$sndKwh = 0;
		}
	    else if($sndKwh > 0)
		{
			$compKwh[$x] = $sndKwh;
			$sndKwh = 0;
		}
	  }
	  else if(count($compKwh) > 0 && $compare_id != 0) 
	  {
	  	$fMat = "";
		/*if($date == 2)
		{
			$fMat = 'YYYY-MM-DD';
		}*/
		$last = $this->hour_model->getLastHour($compare_id, date('Y-m-d H:i:s',$value-($dateInfo['offset'])), $fMat);
		$first = $this->hour_model->getFirstHour($compare_id, date('Y-m-d H:i:s',$value-($dateInfo['offset'])), $fMat);		
		
		
		if($date < 2 && $first->num_rows() == 0 && !$compEnd)
		{
			$totKwh = $this->minute_model->getSumForward($compare_id,  date('Y-m-d H:i:s',$value-3600-($dateInfo['offset'])));
			
			if($totKwh->num_rows() > 0)
			{
				$totKwh = $totKwh->row_array();
				$sndUpd = TRUE;
				if($compType != 1 && $compType != 3)
				{
					$sndKwh = round($totKwh['kwh'],3);
				}
				else
				{
					$sndKwh = round($totKwh['tmp'],3);
				}
				$compData[$x] =  ' "'.$coName.'": ';
				$compKwh[$x] = $sndKwh;
			}
			$compEnd = TRUE; 
		}
		
		if($first->num_rows() > 0 && $last->num_rows() > 0 && !$compEnd)
		{
			if($first->num_rows() > 1)
			{
				$testOne = $first->row_array(0);
				$testTwo = $first->row_array(1);
				
				if((strtotime($testTwo['date'])-strtotime($testOne['date'])) < 3601 && $date < 2)
				{
					$first = $testTwo;
				}
				else {
					$first = $testOne;
				}
			}
			else {
				$first = $first->row_array();
			}
			$last = $last->row_array();

			if($date < 2)
			{
				$hours = sizeof($this->_getHours(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']));
			}
			else if($date == 2)
			{
				$hours = sizeof($this->_getDays(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']))-1;
			}
			else
			{
				$hours = sizeof($this->_getMonths(date($dateInfo['strFormat'],strtotime($last['date'])),date($dateInfo['strFormat'],strtotime($first['date'])),$dateInfo['offset']))-1;	
			}
			if($date != 2)
			{
				if(($last['seconds']-$first['seconds'])!=0)
				{
					if($last['seconds'] > $first['seconds'])
					{
				        $sec = pow(256,3) - $last['seconds'];
				        $sec += $first['seconds'];
					}
					else
					{
						$sec = ($first['seconds']-$last['seconds']);
					}
					
					if($last['wattsecond'] > $first['wattsecond'])
			        {
				          $wsec = pow(256,5) - $last['wattsecond'];
				          $wsec += $first['wattsecond'];
						  $kWh = $wsec/3600000;
			        }
			        else
			        {
			          	 $wsec = $first['wattsecond'] - $last['wattsecond'];
						 $kWh = $wsec/3600000;
			        } 
	
					if($kWh > 20000)
					{
						$kWh = 0;
					}
	
					$sndUpd = TRUE;
					$sndKwh = round($kWh/$hours,3);
					$compData[$x] =  ' "'.$coName.'": ';
					$compKwh[$x] = $sndKwh; 
				}
				
			}
			else
			{
				$sndKwh = $first['kwh']/$hours;	
				$compData[$x] =  ' "'.$coName.'": ';
				$compKwh[$x] = $sndKwh; 
			}
	  	 }
	  }

      $x++;
    }
	
	$x = 0;
	echo '[';
	$oldDate = '0';
	$sum = 0;
	foreach($dateInfo['dateArray'] as $value)
    {
      if(!$firstRec)
	  {
	  	echo ', ';
	  }
	  else {
		  $firstRec = FALSE;
	  }

	  if((date($dateInfo['format'], $value) != date($dateInfo['format'], $oldDate)) || $date == 3)
      {
		echo '{ "date": "'.date($dateInfo['format'],$value).'"';
	  }  
	  else {
		  echo '{ "date": "'.date($dateInfo['format'],$value).'"';	  
	  }
	  
	  
	  if(isset($data[$x]))
	  {
	  	echo ', '.$data[$x].$infoKwh[$x];
		$sum += $infoKwh[$x];
	  }
	  

	  if(isset($compData[$x]))
	  {
	  	echo ', '.$compData[$x].$compKwh[$x];
	  }  
	  echo ' }';
      $oldDate = $value;
      $x++;
    }
	
	echo ']';
  }

   // --------------------------------------------------------------------

   /**
	* Generate JSON for Real-Time graph
	*
	* @access	public
    * @param	int
    * @param	int
    * @param	int
    * @param	int
	* @return	JSON
	*/
  function loadRealTime($chan_id,$compare_id,$interval,$date)
  {
  	$dateInfo['theDate'] = date("Y-m-d",time());
	$userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	$offset = $userTimezone->getOffset();
	
    $format = 'g:i:s A';
	$chanGraph = '[';
	
	$chinfo = $this->channels_model->getChannelInfo($chan_id);
	$chinfo = $chinfo->row_array();
	
	$device = $this->device_model->getSerial($chinfo['device_id']);
	$device = $device->row_array();
	$first = TRUE;
    $x = 0; 
	$maxrow = 0; 
	if($chinfo['ctype'] == 1)
	{
		$cNum = 32 + $chinfo['chnum'];
	}
	else if($chinfo['ctype'] == 2)
	{
		if($device['type'] == "geye")
		{
			$cNum = 40 + $chinfo['chnum'];
		}
		else
		{
			$cNum = 7;	
		}
	}
	else if ($chinfo['ctype'] == 3)
	{
		if($device['type'] == "geye")
		{
			$cNum = 45;
		}
		else
		{
			$cNum = 8;	
		}
	}
	else {
		$cNum = $chinfo['chnum'];
	}
    
	// Read real-time values from file and plot them.
	if(file_exists($device['serial']))
	{
		if (($handle = fopen($device['serial'], "r")) !== FALSE) {
			while ($data = fgetcsv($handle)) {
				if(sizeof($data) >= $cNum)
				{
				    if(!$first)
					{
						$chanGraph .= ',';
					}
					else
					{
						$first = FALSE;	
					}
			        $chanGraph .= '{ "date":"'.date($format, strtotime($data[0])+$offset).'", "'.$chinfo['channel_name'].'":'.$data[$cNum].' }';
				}
		    }
		}
		
		fclose($handle);
	}
    $chanGraph .= ']';
    
    $data['html'] = $chanGraph;
    
    $this->load->view('graphs/index', $data);
  }

   // --------------------------------------------------------------------

   /**
	* Get array of dates based on type
	*
	* @access	private
    * @param	string
    * @param	string
    * @param	int
	* @return	array
	*/
  private function _getDateArray($startDate, $endDate, $type = 0){
  	
	// Determine the type of date range
	if($type == 1)
	{
		$form = "Y-m-d H:00:00";
		$sec = 3600;
	}
	else if($type == 2)
	{
		$form = "Y-m-d";
		$sec = 'P1D';
		
	}
	else if ($type == 3)
	{
		$form = "Y-m";
		$sec = 'P1M';
	}
	else {
		$form = "Y-m-d H:i:00";
		$sec = 60;
	}
	
	// Get the start and end times to loop between, add the appropriate offsets
	$userTimezone = new DateTime($startDate, new DateTimeZone($this->sys['timezone']));
  	$offset = $userTimezone->getOffset();
  	$startDate = strtotime(date($form, strtotime($startDate) + $offset));  
	
	$userTimezone = new DateTime($endDate, new DateTimeZone($this->sys['timezone']));
  	$offset = $userTimezone->getOffset();
 	$endDate = strtotime(date($form, strtotime($endDate) + $offset));  
  
	$dateArray[] = $startDate;  
	$x = 0;
	$currentDate = $startDate;  
	
	// Loop and add dates
	while($currentDate < $endDate){
		if($type < 2)
		{
	    	$currentDate = $currentDate+$sec;
		}
		else {
			$d = new DateTime();
			$d->setTimestamp($currentDate);
			
			$d->add(new DateInterval($sec));
		    $currentDate = strtotime($d->format('c'));
		}  

	    $dateArray[] = $currentDate;
		$x++;  
	}  
	
	return $dateArray;  
  }  

 function _getMinutes($sStartDate, $sEndDate){
	$userTimezone = new DateTime($sStartDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
  	$sStartDate = strtotime(date("Y-m-d H:i:00", strtotime($sStartDate) + $dateInfo['offset']));  
	
	$userTimezone = new DateTime($sEndDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
 	$sEndDate = strtotime(date("Y-m-d H:i:00", strtotime($sEndDate) + $dateInfo['offset']));  
  
	$aDays[] = $sStartDate;  
	$x = 0;
	$sCurrentDate = $sStartDate;  
	
	while($sCurrentDate < $sEndDate){  
	
	    $sCurrentDate = $sCurrentDate+60;  

	    // Add this new day to the aDays array  
	    $aDays[] = $sCurrentDate;
		$x++;  
	}  
	
	// Once the loop has finished, return the  
	// array of days.  
	return $aDays;  
  }  
  
  function _getHours($sStartDate, $sEndDate){
	$userTimezone = new DateTime($sStartDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
    $sStartDate = strtotime(date("Y-m-d H:00:00", strtotime($sStartDate)+$dateInfo['offset']));  
	
	$userTimezone = new DateTime($sEndDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
    $sEndDate = strtotime(date("Y-m-d H:00:00", strtotime($sEndDate)+$dateInfo['offset']));  
    
    $aDays[] = $sStartDate;  
	$x = 0;
	$sCurrentDate = $sStartDate;  
	
	while($sCurrentDate < $sEndDate){
		
	    $sCurrentDate += 3600;
	    // Add this new day to the aDays array  
	    $aDays[] = $sCurrentDate;
		$x++;  
	}
		
    // Once the loop has finished, return the  
    // array of days.  
    return $aDays;  
  }
  
  function _getDays($sStartDate, $sEndDate){
	$userTimezone = new DateTime($sStartDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
	$sStartDate = strtotime(date("Y-m-d", strtotime($sStartDate)+$dateInfo['offset']));
	  
	$userTimezone = new DateTime($sEndDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
	$sEndDate = strtotime(date("Y-m-d", strtotime($sEndDate)+$dateInfo['offset']));  
	  
	$aDays[] = $sStartDate;  
	$sCurrentDate = $sStartDate; 
	
	while($sCurrentDate < $sEndDate){  
		$d = new DateTime();
		$d->setTimestamp($sCurrentDate);
		
		$d->add(new DateInterval('P1D'));
	    $sCurrentDate = strtotime($d->format('c'));

	    // Add this new day to the aDays array  
	    $aDays[] = $sCurrentDate;
	}
	
	// Once the loop has finished, return the  
	// array of days.  
	return $aDays;  
  }
  
  function _getMonths($sStartDate, $sEndDate){
	$userTimezone = new DateTime($sStartDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset();
    $sStartDate = strtotime(date("Y-m", strtotime($sStartDate)+$dateInfo['offset']));  
	 
	$userTimezone = new DateTime($sEndDate, new DateTimeZone($this->sys['timezone']));
  	$dateInfo['offset'] = $userTimezone->getOffset(); 
    $sEndDate = strtotime(date("Y-m", strtotime($sEndDate)+$dateInfo['offset']));  
  
    $aDays[] = $sStartDate;  

    $sCurrentDate = $sStartDate;  

    while($sCurrentDate < $sEndDate){  
		$d = new DateTime();
		$d->setTimestamp($sCurrentDate);
		
		$d->add(new DateInterval('P1M'));
	    $sCurrentDate = strtotime($d->format('c'));
		
      // Add this new day to the aDays array  
      $aDays[] = $sCurrentDate;  
    }  
  
    // Once the loop has finished, return the  
    // array of days.  
    return $aDays;  
  }
  
     
   // --------------------------------------------------------------------

   /**
	* Get power statistics between two dates
	*
	* @access	private
	* @return	array
	*/
  private function _getChannelPower($date, $begin, $end, $chan_id, $compare)
  {
    $result['usedTkWh'] = 0;
    $result['usedTCost'] = 0;
    $result['avgWatt'] = 0;
	$result['minWatt'] = 0;
	$result['maxWatt'] = 0;
	
	// If main panel, use those queries
    if($chan_id == 0)
    {
      if($date < 1)
      {
	      $powInfo = $this->minute_model->getMainPower($begin, $end);
	      $powInfo = $powInfo->row_array();
	  }
	  else
	  {
		  $powInfo = $this->hour_model->getMainPower($begin, $end);
	      $powInfo = $powInfo->row_array();
	  }

      $result['usedTkWh'] = $powInfo['kwh'];
      $result['avgWatt'] = $powInfo['watt'];
      $result['usedTCost'] = $powInfo['totalcost'];	
	  $result['minWatt'] = $powInfo['minWatt'];
	  $result['maxWatt'] = $powInfo['maxWatt'];
    }
    else
    {
      if($date < 1)  // If date is minute, use data based on minute data
      {
      	$cha = $this->channels_model->getChannelInfo($chan_id);
	    $cha = $cha->row_array();
      
		if($cha['ctype'] == 1)
		{
			$pow = $this->minute_model->getTempVals($begin, $end, $chan_id);
		 	$pow = $pow->row_array();
		}
		else {
			$pow = $this->minute_model->getSumPower($begin, $end, $chan_id);
		 	$pow = $pow->row_array();
		}
      }
      else  // Use hour data
      {
        $pow = $this->hour_model->getSumPower($begin, $end, $chan_id);
		$pow = $pow->row_array();
      }
	 

      $result['usedTCost'] = $pow['totalcost'];
      $result['usedTkWh'] = $pow['kwh'];
      $result['avgWatt'] = $pow['watt'];
	  $result['minWatt'] = $pow['minwatt'];
	  $result['maxWatt'] = $pow['maxwatt'];
    }
    return $result;
  }
     
   // --------------------------------------------------------------------

   /**
	* Get the date information needed by almost every function
	*
	* @access	private
	* @return	array
	*/

  private function _getDates($date, $interval,  $extras = true)
  {
  	$dateInfo['today'] = '';
  	if($date == 0)  // Daily
    {
      $dTime = mktime(0, 0, 0, date("m") , date("d")-$interval, date("Y"));
      $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dTime);
	  $userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
      
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dTime);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, date("m") , date("d")-$interval, date("Y")));
	  $dateInfo['titleBegin'] = date('D, M j, Y', $dTime);
	  $dateInfo['today'] = date('m/d/Y', mktime(0, 0, 0, date("m") , date("d"), date("Y")));
	  
	  if($extras)
	  {
		  $dateInfo['strFormat'] =  'Y-m-d H:i:00';
	      $dateInfo['format'] = 'g:i a';
	      $dateInfo['indVal'] = 60;
		  date_default_timezone_set('UTC');
		  $dateInfo['genIndex'] = strtotime($dateInfo['begin'])+1-60;
		  date_default_timezone_set($this->sys['timezone']);
	      $dateInfo['dateArray'] = $this->_getDateArray($dateInfo['begin'],$dateInfo['end']);
	  }
    } 
    elseif($date == 1)  // Weekly
    {
      $dateInfo['start'] = mktime(0, 0, 0, date('m'), date('d')-date('w')-($interval*7)+1, date('Y'));
      
	  $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
	  $userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
	  
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', $dateInfo['start'] + (7 * 24*60*60));
	  
	  $dateInfo['titleBegin'] = date('M j, Y', $dateInfo['start']);
	   
	  if($extras)
	  {
	  	$dateInfo['graphForm'] = 'YYYY-MM-DD HH24:00:00';
	  	$dateInfo['strFormat'] =  'Y-m-d H:00:00';
      	$dateInfo['format'] = 'D, g a';
	  	$dateInfo['indVal'] = 3600;
		date_default_timezone_set('UTC');
	  	$dateInfo['genIndex'] = strtotime($dateInfo['begin'])+1-3600;
		date_default_timezone_set($this->sys['timezone']);
      	$dateInfo['dateArray'] = $this->_getDateArray($dateInfo['begin'],$dateInfo['end'],1);
      }
    }
    elseif($date == 2)  // Monthly
    {
      $dateInfo['start'] = mktime(00, 00, 01, date("m")-$interval, $this->sys['month_start'], date("Y"));
      
	  $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
	  $userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
      
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, date("m")-$interval+1, $this->sys['month_start']-1, date("Y")));
	  
	  $dateInfo['titleBegin'] = date('M Y', mktime(0, 0, 0, date("m")-$interval, $this->sys['month_start'], date("Y")));
	  
	  if($extras)
	  {
	  	$dateInfo['graphForm'] =  'YYYY-MM-DD';
	  	$dateInfo['strFormat'] =  'Y-m-d';
      	$dateInfo['format'] = 'D, M j';
	  	$dateInfo['indVal'] = 86400;
		date_default_timezone_set('UTC');
	  	$dateInfo['genIndex'] = strtotime($dateInfo['begin'])+1-86400;
		date_default_timezone_set($this->sys['timezone']);
      	$dateInfo['dateArray'] = $this->_getDays($dateInfo['begin'],$dateInfo['end'],$dateInfo['offset']);
      }
    }
    else  // Yearly
    {
      $dateInfo['start'] = mktime(23, 59, 59, 1, $this->sys['month_start']-1, date("Y")-$interval);
      
	  $dateInfo['theDate'] = date("Y-m-d 00:00:01",$dateInfo['start']);
	  $userTimezone = new DateTime($dateInfo['theDate'], new DateTimeZone($this->sys['timezone']));
  	  $dateInfo['offset'] = $userTimezone->getOffset();
      
      $dateInfo['begin'] = gmdate('Y-m-d H:i:s', $dateInfo['start']);
      $dateInfo['end'] = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, 1, $this->sys['month_start'], date("Y")-($interval-1)));
      
	  $dateInfo['titleBegin'] = date('Y', mktime(0, 0, 0, 1, 1, date("Y")-$interval));
	   
	  if($extras)
	  {
	  	$dateInfo['graphForm'] =  'YYYY-MM';
	  	$dateInfo['strFormat'] =  'Y-m';
      	$dateInfo['format'] = 'M';
	  	$dateInfo['indVal'] = 2628000;
		date_default_timezone_set('UTC');
	  	$dateInfo['genIndex'] = strtotime($dateInfo['begin'])+1-2628000;
		date_default_timezone_set($this->sys['timezone']);
      	$dateInfo['dateArray'] = $this->_getMonths($dateInfo['begin'],$dateInfo['end'],$dateInfo['offset']);
	  }
    }
	
	return $dateInfo;
  }
}