<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * Virtual Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	Setup
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class Virtual extends MX_Controller {
   
  /**
   * Constructor
   *
   * @access	public
   */
  function Virtual()
  {
    parent::__construct();
    $this->load->library('form_validation');
    $this->load->helper('form');
    $this->load->helper('url');
    $this->load->model('device_model');
    $this->load->model('virtual_model');
    $this->load->model('channels_model');
	$this->load->library('session');
	if($_SERVER['REMOTE_ADDR'] != "::1")
	{
	    if (!$this->session->userdata('admin'))
	    {
	        redirect('/pages/loads/');
	    }
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Load a list of virtual channels
	*
	* @access	public
	* @return	void
	*/
  function index()
  {
    $virtual = $this->device_model->getDevices('virtual');
	if($virtual->num_rows() > 0)
	{
		$virtual = $virtual->row_array();
		$virtual['channels'] = $this->channels_model->getVirtChannels($virtual['device_id']);
	}
	else {
		$virtual = array();
		$virtual['serial'] = -1;
	}
	
    $this->load->view('home/settings/virtual', $virtual);
  }
  
   // --------------------------------------------------------------------

   /**
	* Add a new virtual device
	*
	* @access	public
	* @return	void
	*/
  function AddDevice()
  {
  	$this->form_validation->set_rules('alias','Device Serial','min_length[5]|max_length[8]|required');
	if($this->form_validation->run() == FALSE)
    {
       $this->load->view('home/settings/addvirtual');  
	}
	else
	{
		$this->device_model->InsertVirtualDevice(set_value('alias'));
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Add a new virtual channel
	*
	* @access	public
	* @return	void
	*/
  function AddChannel($dev_id)
  {
  	$this->form_validation->set_rules('channel_name','Channel Name','min_length[1]|max_length[30]|required');
	if($this->form_validation->run() == FALSE)
    {
       $data['dev_id'] = $dev_id;
       $this->load->view('home/settings/addvirtchannel', $data);  
	}
	else
	{
		$this->channels_model->InsertChannel(set_value('channel_name'), $dev_id, 0, 0, $this->_random_colour(), '', 5);
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	}
  }
  
   // --------------------------------------------------------------------

   /**
	* Modify a virtual channel
	*
	* @access	public
	* @return	void
	*/
  function ManageChannel($channel_id)
  {
  	$this->form_validation->set_rules('colour', 'Colour',  'required|min_length[1]|max_length[6]');
    $this->form_validation->set_rules('channel_name', 'Channel Name',  'required|min_length[3]|max_length[25]');
    $this->form_validation->set_rules('channel_id', 'Channel ID',  'required');
    
    if($this->form_validation->run() == FALSE)
    {
      if($channel_id != 0) 
      {
        $channel = $this->channels_model->getChannelInfo($channel_id);
        $channel = $channel->row_array();
        
        $data['channel_name'] = $channel['channel_name'];
        $data['colour'] = $channel['colour'];
        $data['channel_id'] = $channel_id;
      }
      else
      {
        echo '<script>var $ = parent.$; $().colorbox.close();</script>';
      }
      
      $this->load->view('home/settings/managevirtual', $data);  
    }
    else
    {
    	if($channel_id > 0)
		{
	    	$update = array("colour" => set_value('colour'), "channel_name" => set_value('channel_name'));
	        $this->channels_model->UpdateChannel($channel_id, $update);
		}
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
    }
  }
  
   // --------------------------------------------------------------------

   /**
	* Remove a virtual channel
	*
	* @access	public
	* @return	void
	*/
  function RemoveDevice()
  {
  	$this->form_validation->set_rules('serial','Device Serial','min_length[2]|max_length[8]|required');
	
	if($this->form_validation->run() == FALSE)
    {
       $data['devices'] = $this->device_model->getDevices('virtual');
       $this->load->view('home/settings/removedevice', $data);  
	}
	else
	{
		$this->device_model->DeleteDevice(set_value('serial'));
		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	}
  }
	
   // --------------------------------------------------------------------

   /**
	* Process ECM-1240 packet
	*
	* @access	public
	* @return	void
	*/
  function RemoveChannel($channel_id)
  {
  	$this->channels_model->RemoveVirtChannel($channel_id);
  	redirect('/pages/virtual');
  }

   // --------------------------------------------------------------------

   /**
	* Modify channels in and not in virtual channels
	*
	* @access	public
	* @return	void
	*/
  function GetVirtChannels($virtual_id)
  {
    $data['ch_in'] = $this->virtual_model->GetChannelsModule($virtual_id);
    $data['ch_not'] = $this->virtual_model->GetChannelsNot($virtual_id);
    $data['virtual_id'] = $virtual_id;
    $this->load->view('home/settings/virtchannels',$data);
  }

   // --------------------------------------------------------------------

   /**
	* Add channel to virtual channel
	*
	* @access	public
	* @return	void
	*/
  function AddVChannel($virtual_id, $channel_id)
  {
    $this->virtual_model->AddChannel($virtual_id, $channel_id);
    redirect(base_url().'index.php/pages/virtual/GetVirtChannels/'.$virtual_id);
  }
  
   // --------------------------------------------------------------------

   /**
	* Remove channel from virtual channel
	*
	* @access	public
	* @return	void
	*/
  function RemoveVChannel($virtual_id,$virtchan_id)
  {
    $this->virtual_model->RemoveChannel($virtchan_id);
    redirect(base_url().'index.php/pages/virtual/GetVirtChannels/'.$virtual_id);
  }
  
   // --------------------------------------------------------------------

   /**
	* Random colour generator
	*
	* @access	private
	* @return	string
	*/
  private function _random_colour()
  {
	  mt_srand((double)microtime()*1000000);
	  $c = '';
	  while(strlen($c)<6){
	      $c .= sprintf("%02X", mt_rand(0, 255));
	  }
	  return $c;
  }
}

