<?php
class Datalog_model extends CI_Model {

  function Datalog_model()
  {
    parent::__construct();
	$this->load->database('default', true);
  }
  
  function getVirtDevices()
  {
  	$this->db->select('device_id, serial, net, type, alias, gransec, minsec, hoursec, grandate, mindate, hourdate');
    $this->db->where('type','virtual');
    $query = $this->db->get('devices');
    
    return $query;
  }
  
  
  function getTemp($dev, $chnum = 0)
  {
    $this->db->select('channel_id, channel_name, colour, hide, granws, minws, hourws, views, pulse_unit');
	$this->db->where('device_id',$dev);
	if($chnum != 0)
	{
		$this->db->where('chnum', $chnum);
	}
	$this->db->where('ctype = 1');
	$this->db->order_by('chnum');
    return $this->db->get('channel');
  }
  
  function getLastMinute($virtual_id, $date)
  {
  	$this->db->select('sum(watt) as watt, sum(kwh) as kwh, sum(totalcost) as totalcost, sum(cost) as cost, max(date) as date');
  	$this->db->join('virtual_chan', 'virtual_chan.channel_id = minutedata.channel_id', 'inner'); 
  	$this->db->where('virtual_chan.virtual_id', $virtual_id); 
  	$this->db->where('date', $date);
	
    return $this->db->get('minutedata');
  }
  
  function getLastHour($virtual_id, $date)
  {
  	$this->db->select('sum(watt) as watt, sum(kwh) as kwh, sum(totalcost) as totalcost, sum(cost) as cost, max(date) as date');
  	$this->db->join('virtual_chan', 'virtual_chan.channel_id = hourdata.channel_id', 'inner'); 
  	$this->db->where('virtual_chan.virtual_id', $virtual_id); 
  	$this->db->where('date', $date);
	
    return $this->db->get('hourdata');
  }
  
  function getVirtChannels()
  {
    $this->db->select('channel_id, channel_name, colour, hide, granws, minws, hourws, views, pulse_unit');
	$this->db->where('ctype = 5');
	$this->db->order_by('chnum');
    return $this->db->get('channel');
  }
  
  function UpdateDevice($devid, $date, $sec)
  {
    $this->db->set('grandate', $date);
    $this->db->set('mindate', $date);
    $this->db->set('hourdate', $date);
	$this->db->set('gransec', $sec);
	$this->db->set('minsec', $sec);
	$this->db->set('hoursec', $sec);
	$this->db->where('device_id', $devid);
	$this->db->update('devices');
  }
  
  function GetInfo()
  {
  	$this->db->select('flat_rate, month_start, holiday, sat, sun, prov, billing_type, timezone');
    return $this->db->get('sysinfo');
  }
  
  function GetCommandsDevice($device_id)
  {
  	$this->db->select('command');
	$this->db->where('device_id', $device_id);
    return $this->db->get('device_commands');
  }
  
  function GetTieredRates()
  {
  	$this->db->order_by('max_kwh', 'ASC');
    $query = $this->db->get('tiers');
    return $query;
  }
  
  function GetTOURates()
  {
    $this->db->order_by('start_date DESC, end_time ASC');
    $query = $this->db->get('timeofuse');
    return $query;
  }
  
  function GetMainPower($begin, $end)
  {
  	$this->db->select('sum(kwh) as kwh');
    $this->db->join('channel', 'channel.channel_id = minutedata.channel_id', 'inner');  
    $this->db->where("date BETWEEN ".$this->db->escape($begin)." AND ".$this->db->escape($end)." ");
	$this->db->where('main',1);
	$this->db->where('ctype',0);
	$this->db->where('netchannel_id',0);
    return $this->db->get('minutedata');
  }

  function getDevice($serial)
  {
    $this->db->select('device_id, serial, net, type, alias, gransec, minsec, hoursec, grandate, mindate, hourdate');
    $this->db->where('serial',$serial);
    $query = $this->db->get('devices');
    
    return $query;
  }
  
  function GetDeviceChannels($devid, $ctype = 10)
  {
    $this->db->select('channel_id, main, pulse, channel_name, netchannel_id, granws, minws, hourws');
    $this->db->where('device_id', $devid);
	if($ctype != 10)
	{
		$this->db->where('ctype', $ctype);
		$this->db->where('netchannel_id', 0);
	}
    $this->db->order_by('chnum');
    return $this->db->get('channel');
  }
    
  function GetECMChan($devid)
  {
    $this->db->select('channel_id, main, pulse, channel_name, netchannel_id, granws, minws, hourws');
    $this->db->where('device_id', $devid);
	$this->db->where('netchannel_id', 0);
	$this->db->where('ctype < 3');
    $this->db->order_by('chnum');
    return $this->db->get('channel');
  }
  
  function GetECMChannels($devid)
  {
    $this->db->select('channel_id, main, pulse, channel_name, netchannel_id, granws, minws, hourws');
    $this->db->where('device_id', $devid);
	$this->db->where('ctype < 3');
    $this->db->order_by('chnum');
    return $this->db->get('channel');
  }
  
  function GetNetDevice($devid)
  {
    $this->db->select('channel_id, chnum, main, pulse, channel_name, netchannel_id, granws, minws, hourws');
    $this->db->where('netchannel_id >', 0);
    $this->db->where('device_id',$devid);
	$this->db->order_by('chnum');
    return $this->db->get('channel');
  }

  function GetChannel($chid)
  {
    $this->db->select('granws, minws, hourws');
    $this->db->where('channel_id',$chid);
    return $this->db->get('channel');
  }
    
  function GetGreenPulses($dev, $chnum = 0)
  {
    $this->db->select('channel_id, pulse, pulse_unit, channel_name, hide, netchannel_id, colour, granws, minws, hourws');
	$this->db->where('device_id',$dev);
	if($chnum != 0)
	{
		$this->db->where('chnum', $chnum);
	}
	$this->db->where('ctype = 2');
	$this->db->order_by('chnum');
    return $this->db->get('channel');
  }

  function GetAvgTemp ($begin, $end, $channelid)
  {
    $this->db->select('avg(kwh) as kwh');
    $this->db->join('channel', 'channel.channel_id = minutedata.channel_id', 'inner');  
    $this->db->where("channel.channel_id",$channelid);
    $this->db->where("date BETWEEN ".$this->db->escape($begin)." AND ".$this->db->escape($end)." ");
    $query = $this->db->get('minutedata');
    return $query->row_array();
  } 

  function InsertChannel($chalias, $devid, $chnum, $netchid, $colour, $pulse, $WS, $ctype = 0)
  {
  	
    $data = array(
               'main' => 0,
               'channel_name' => $chalias,
               'pulse' => $pulse,
               'device_id' => $devid,
               'chnum' => $chnum,
               'colour' => $colour,
               'ctype' => $ctype,
               'hide' => 0,
               'netchannel_id' => $netchid,
               'granws' => $WS,
               'minws' => $WS,
               'hourws' => $WS,
               'views' => 0
            );

    $this->db->insert('channel', $data); 
  }
}
?>