<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * Datalog Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	DataLog
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class DataLog extends MX_Controller {
   private $sysinfo, $now, $devInfo;
   
  /**
   * Constructor
   *
   * @access	public
   */
  function DataLog()
  {
    parent::__construct();
	$this->load->database('default', true);
    $this->load->model('datalog_model');
    $this->load->helper('costs');
	
	// Get values needed throughout the script
    $this->sysinfo = $this->datalog_model->getInfo();
    $this->sysinfo = $this->sysinfo->row_array();
    date_default_timezone_set($this->sysinfo["timezone"]);
    $this->now = gmdate('Y-m-d H:i:s');
	
    ignore_user_abort(true); 
  }
  
   // --------------------------------------------------------------------

   /**
	* Process ECM-1240 packet
	*
	* @access	public
	* @return	void
	*/
  function log1240($wsArray, $polArray, $v, $dev, $sec, $chAmt)
  {
  	// Check if the device exists
    $devList = $this->datalog_model->GetDevice($dev);
    if($devList->num_rows() > 0)
    {
      $this->devInfo = $devList->row_array();
	  
	  // Make sure there's channels
      $chList = $this->datalog_model->GetECMChan($this->devInfo['device_id'],0);
      if($chList->num_rows() > 0)
      {
      	// Check to make sure it's a new packet	
        if(strtotime($this->now) > strtotime($this->devInfo['grandate']))
		{	
			$sql = '';
			
		    $sql .= $this->_startRecord($chList, $chAmt, $sec, $wsArray, $polArray, 4);
		    
			$volt = $this->datalog_model->GetDeviceChannels($this->devInfo['device_id'],3);
			$volt = $volt->row_array();
			
			$ch = fopen($this->devInfo['serial'], 'a');
			fwrite($ch, ','.$v."\n");
			fclose($ch);
			
			if(mktime(gmdate("H"), gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['mindate']))
			{
				$sql .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
	  				VALUES (".$volt['channel_id'].",'".gmdate('Y-m-d H:i:00')."',".$v.",".$sec.",".$v.",".$v.",0,0); ";					
			}
			
			$sql .= $this->_logvirtual();
			
			
		    if($sql != '')
			{
  				$sqlCmds = explode(';',$sql);
				
  				$this->db->query("BEGIN;");
				for($x=0;$x<sizeof($sqlCmds);$x++)
				{
					if(strlen($sqlCmds[$x]) > 5)
					{
						$this->db->query($sqlCmds[$x]);
					}
				}
				$this->db->query("COMMIT;");
			}
		}
      }
      else
      {
      	$this->datalog_model->UpdateDevice($this->devInfo['device_id'],$this->now,$sec);

        for($x=0;$x<$chAmt;$x++)
        {
          $y = $x + 1;
          $this->datalog_model->InsertChannel('Channel '.$y, $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 0, $wsArray[$x], 0);
        }
		
		$y = $y + 1;
        $this->datalog_model->InsertChannel('Voltage', $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 0, 0, 3);
      }
    }
  }
  
   // --------------------------------------------------------------------

   /**
	* Process GreenEye packet
	*
	* @access	public
	* @return	void
	*/
  function loggeye($wsArray, $polArray, $v, $dev, $sec, $chAmt, $pulse, $temp)
  {
    $devList = $this->datalog_model->getDevice($dev);
    if($devList->num_rows() > 0)
    {
      $this->devInfo = $devList->row_array();
      
      $chList = $this->datalog_model->GetDeviceChannels($this->devInfo['device_id'],0);
      if($chList->num_rows() > 0)
      {	
        if(strtotime($this->now) > strtotime($this->devInfo['grandate']))
		{
			// Update granular date right away in case another packet comes in before it finishes processing	
			$this->db->query("UPDATE devices SET grandate = '".$this->now."' where device_id = ".$this->devInfo['device_id']);	
			$sql = '';
			
			// Get the SQL for the channels
	        $sql .= $this->_startRecord($chList, $chAmt, $sec, $wsArray, $polArray, 5);   
			
			$change = 0;
			$y = 0;
			
			// Generate the temperature records
			for($x=0;$x<sizeof($temp);$x++)
			{
				if($temp[$x] != '')
				{
	          		$y = $x + 1;
					$tempChannel = $this->datalog_model->getTemp($this->devInfo['device_id'], $y);
					if($tempChannel->num_rows() == 0)
					{
						$this->datalog_model->InsertChannel('Temp. '.$y, $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 0, 0, 1);
					}
					else
					{
						$tmpVal = round($temp[$x],1);
						$tempChannel = $tempChannel->row_array();
						$ch = fopen($this->devInfo['serial'], 'a');
						fwrite($ch, ','.round($tmpVal,1));
						fclose($ch);
						
						if(mktime(gmdate("H"), gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['mindate']))
						{
							if($tempChannel['minws'] != 0)
							{
								$avgTemp = round($tempChannel['hourws']/$tempChannel['minws'],3);
							}
							else
							{
								$avgTemp = 0;
							}
							$sql .= 'UPDATE channel SET hourws = '.$tmpVal.', minws = 1 where channel_id = '.$tempChannel['channel_id'].';';
							if($avgTemp < 1000)
							{
								$sql .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
					  				VALUES (".$tempChannel['channel_id'].",'".gmdate('Y-m-d H:i:00')."',0,0,".$avgTemp.",".$avgTemp.",0,0); ";
							}
						}
						else {
							$sql .= 'UPDATE channel SET hourws = hourws + '.$tmpVal.', minws = minws + 1 where channel_id = '.$tempChannel['channel_id'].';';	
						}
						
						if(mktime(gmdate("H"), 0, 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['hourdate']))
						{
							$avgTmp = $this->datalog_model->GetAvgTemp($this->devInfo['hourdate'], $this->now, $tempChannel['channel_id']);
							
							if($avgTmp['kwh'] < 1000)
							{
								$sql .= "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
					  				VALUES (".$tempChannel['channel_id'].",'".gmdate('Y-m-d H:00:00')."',0,0,".round($avgTmp['kwh'],3).",".round($avgTmp['kwh'],2).",0,0); ";
					  		}
						}
					}
				}
			}
			
			// Generate the pulse records
			for($x=0;$x<sizeof($pulse);$x++)
			{
				if($pulse[$x] != '')
				{
	          		$y = $x + 1;
					$pChan = $this->datalog_model->GetGreenPulses($this->devInfo['device_id'],$y);
					
					if($pChan->num_rows() == 0)
					{
						$this->datalog_model->InsertChannel('Pulse '.$y, $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 1, $pulse[$x], 2);
					}
					else
					{
						$pChan = $pChan->row_array();
						$vals = $this->_wattSecDiff($pChan['granws'], $pulse[$x], $this->devInfo['gransec'], $sec, 1, 5);
												
						if($vals[0] != 0)
						{
							$watt = round((($vals[1]*$pChan['pulse'])/$vals[0])*60,2);
						}
						else
						{
							$watt = 0;	
						}
						
						$ch = fopen($this->devInfo['serial'], 'a');
						fwrite($ch, ','.$watt);
						fclose($ch);
						
						$update = 'UPDATE channel SET granws = '.$pulse[$x];
						
						if(mktime(gmdate("H"), gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['mindate']))
						{
							
							$vals = $this->_wattSecDiff($pChan['minws'], $pulse[$x], $this->devInfo['minsec'], $sec, 1, 5);
							
							if($vals[0] != 0)
							{
								$watt = round(($vals[1]*$pChan['pulse'])/$vals[0]*60,3);
							}
							else
							{
								$watt = 0;	
							}
							
							if($watt > 10000)
							{
								$watt = 0;
							}
							
							
							$sql .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
				  					VALUES (".$pChan['channel_id'].",'".gmdate('Y-m-d H:i:00')."',".$pulse[$x].",".$sec.",".$watt.",".round($vals[1]*$pChan['pulse'],2).",0,0); ";
														
							$update .= ', minws = '.$pulse[$x];		
						}
						
						if(mktime(gmdate("H"), 0, 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['hourdate']))
						{
							$vals = $this->_wattSecDiff($pChan['hourws'], $pulse[$x], $this->devInfo['hoursec'], $sec, 1, 5);

							if($vals[0] != 0)
							{
								$watt = round(($vals[1]/$vals[0])*3600,3);
							}
							else
							{
								$watt = 0;	
							}
							
							if($watt > 10000)
							{
								$watt = 0;
							}
							
							$sql .= "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
				  					VALUES (".$pChan['channel_id'].",'".gmdate('Y-m-d H:00:00')."',".$pulse[$x].",".$sec.",".$watt.",".round($vals[1]*$pChan['pulse'],2).",0,0); ";
							
							$update .= ', hourws = '.$pulse[$x];
						}
					
						$update .= ' where channel_id = '.$pChan['channel_id'].'; ';
						
						$sql .= $update;
					}

				}
			}		
							
			$volt = $this->datalog_model->GetDeviceChannels($this->devInfo['device_id'],3);
			
			// Generate the voltage records
			if($volt->num_rows() > 0)
			{
				$volt = $volt->row_array();
				$ch = fopen($this->devInfo['serial'], 'a');
				fwrite($ch, ','.$v."\n");
				fclose($ch);
				
				if(mktime(gmdate("H"), gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['mindate']))
				{
					$avgVolt = round($volt['hourws']/$volt['minws'],3);
					$sql .= 'UPDATE channel SET hourws = '.$v.', minws = 1 where channel_id = '.$volt['channel_id'].';';
					if($avgVolt < 1000)
					{
						$sql .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
			  				VALUES (".$volt['channel_id'].",'".gmdate('Y-m-d H:i:00')."',0,0,".$avgVolt.",".$avgVolt.",0,0); ";
					}
				}
				else {
					$sql .= 'UPDATE channel SET hourws = hourws + '.$v.', minws = minws + 1 where channel_id = '.$volt['channel_id'].';';	
				}
				
				if(mktime(gmdate("H"), 0, 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($this->devInfo['hourdate']))
				{
					$avgVolt = $this->datalog_model->GetAvgTemp($this->devInfo['hourdate'], $this->now, $volt['channel_id']);
					
					if($avgVolt['kwh'] < 1000)
					{
						$sql .= "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
			  				VALUES (".$volt['channel_id'].",'".gmdate('Y-m-d H:00:00')."',0,0,".round($avgVolt['kwh'],3).",".round($avgVolt['kwh'],2).",0,0); ";
			  		}
				}
			}
			
			$sql .= $this->_logvirtual();
			
			
			// Execute the final SQL string		
			if($sql != '')
			{
				$sqlCmds = explode(';',$sql);
  				$this->db->query("BEGIN;");
				for($x=0;$x<sizeof($sqlCmds);$x++)
				{
					if(strlen($sqlCmds[$x]) > 5)
					{
						$this->db->query($sqlCmds[$x]);
					}
				}
				$this->db->query("COMMIT;");
				
				
			}			
    	}
	  }
      else
      {
      	// If no channels exist, insert the channels for the device
      	$this->datalog_model->UpdateDevice($this->devInfo['device_id'],$this->now,$sec);
        for($x=0;$x<$chAmt;$x++)
        {
          $y = $x + 1;
          $this->datalog_model->InsertChannel('Channel '.$y, $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 0, $wsArray[$x], 0);
        }
		$y = $y + 1;
        $this->datalog_model->InsertChannel('Voltage', $this->devInfo['device_id'], $y, 0, $this->_random_colour(), 0, 0, 3);
      }
    }
  }
  
   // --------------------------------------------------------------------

   /**
	* Process virtual channels
	*
	* @access	private
	* @return	string
	*/
  private function _logvirtual()
  {
  	$sql = '';
  	
  	$virtDevs = $this->datalog_model->getVirtDevices();
	if($virtDevs->num_rows() > 0)
    {
      $virtDevs = $virtDevs->row_array();
      
      $virtChans = $this->datalog_model->getVirtChannels();
	  if($virtChans->num_rows() > 0)
      {	
        if(strtotime($this->now) > strtotime($virtDevs['grandate']))
		{
			$updSQL = "UPDATE devices SET grandate = '".$this->now."'";
			
			$updMin = FALSE;
			$updHour = FALSE;
			$minDate = '';
			$hourDate = '';
			
			// Check the dates, make sure the virtual device hasn't already been updated
			// in the case of virtual channels that span several devices
			if(mktime(gmdate("H"), gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($virtDevs['mindate'])+60)
			{
				$updMin = TRUE;
				$minDate = gmdate('Y-m-d H:i:00',mktime(date("H"), date("i")-1, 0, date("m"), date("d"), date("Y")));
				$updSQL .= ", mindate = '".$minDate."'";
			}	
			
			if(mktime(gmdate("H")-1, gmdate("i"), 0, gmdate("m"), gmdate("d"), gmdate("Y")) > strtotime($virtDevs['hourdate'])+60)
			{
				$updHour = TRUE;
				$hourDate = gmdate('Y-m-d H:i:00',mktime(date("H"), 0, 0, date("m"), date("d"), date("Y")));
				$updSQL .= ", hourdate = '".$hourDate."'";
			}
			
			$updSQL .= " where device_id = ".$virtDevs['device_id'];
			
			$this->db->query($updSQL);
			
			// Loop through and process all virtual channels
			foreach($virtChans->result() as $virt)
			{	
				if($updMin)
				{
					$minDat = $this->datalog_model->getLastMinute($virt->channel_id, $minDate);
					if($minDat->num_rows() > 0)
					{
						$minDat = $minDat->row_array();
						if($minDat['watt'] > 0)
						{
							$sql .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
			  					VALUES (".$virt->channel_id.",'".$minDate."',0,0,".$minDat['watt'].",".$minDat['kwh'].",".$minDat['cost'].",".$minDat['totalcost']."); ";
						}
					}
				}
				
				if($updHour)
				{
					$hourDat = $this->datalog_model->getLastHour($virt->channel_id, $hourDate);
				
					if($hourDat->num_rows() > 0)
					{
						$hourDat = $hourDat->row_array();
						if($hourDat['watt'] > 0)
						{
							$sql .= "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
		  						VALUES (".$virt->channel_id.",'".$hourDate."',0,0,".$hourDat['watt'].",".$hourDat['kwh'].",".$hourDat['cost'].",".$hourDat['totalcost']."); ";
						}
					}
				}
			}
			
		}
	  }
	}
	
	return $sql;
  }
  
   // --------------------------------------------------------------------

   /**
	* Start of the packet processing
	*
	* @access	private
    * @param	object
    * @param	int
    * @param	int
    * @param	array
    * @param	array
    * @param	int
    * @param	int
	* @return	string
	*/
  private function _startRecord($chInfo, $chAmt, $sec, $wsArray, $polArray, $wsSize)
  {
	$sql = '';
	
	$minDate = gmdate('Y-m-d H:i:00');
	$hourDate = gmdate('Y-m-d H:00:00');
    $num = $this->datalog_model->GetNetDevice($this->devInfo['device_id']);
	
	// Log NET metered channels
    if($num->num_rows > 0)
	{
		$updates = ''; 
		$updateNet = FALSE;
        foreach($num->result() as $net)
        {
          // Update the channel based on the new values
	  	  $updates .= 'UPDATE channel SET granws = '.$polArray[$net->chnum-1];
          
          // Make sure there's already a value in granular wattseconds
          if($net->granws != '')          					
		  {
		  	  $updateNet = TRUE;
			  
			  // Make sure at least a minute has elapsed since the last minute record
			  if(strtotime($minDate) > strtotime($this->devInfo['mindate']))
			  {
			  	
				// Generate SQL statement
			  	$sql .= $this->_processNet($net, $wsArray[$net->chnum-1], $polArray[$net->chnum-1], $net->netchannel_id, $this->devInfo['mindate'], $sec, 0);
				$updates .= ', minws = '.$polArray[$net->chnum-1];
			  }
			  			  
			  // Make sure an hour has elapsed since the last hour record
			  if(strtotime($hourDate) > strtotime($this->devInfo['hourdate']))
			  {
			  	// Generate SQL statement
			  	$sql .= $this->_processNet($net, $wsArray[$net->chnum-1], $polArray[$net->chnum-1], $net->netchannel_id, $this->devInfo['hourdate'], $sec, 1);
				$updates .= ', hourws = '.$polArray[$net->chnum-1];
			  }
		  }
		  else {
		  	
			 // If it's the first record, default the wattseconds values
		  	 $updateNet = TRUE;
			 $updates .= ', minws = '.$polArray[$net->chnum-1]; 
			 $updates .= ', hourws = '.$polArray[$net->chnum-1];
		  }
		  
		  $updates .= " WHERE channel_id = ".$net->channel_id.";";
        }

		// Update the NET metered channels
		if($updateNet)
		{	
		  $updates = explode(';',$updates);

		  $this->db->query("BEGIN;");
		  for($x=0;$x<sizeof($updates);$x++)
		  {
			  if(strlen($updates[$x]) > 5)
			  {
			  	 $this->db->query($updates[$x]);
			  }
		  }
		  $this->db->query("COMMIT;");
		}
	 }

	// Process the rest of the channels	
	$sql .= $this->_processRecord($chInfo, $chAmt, $sec, $wsArray, $polArray, $wsSize, $minDate, $hourDate);

	return $sql;
  }
  
   // --------------------------------------------------------------------

   /**
	* Generate SQL for insertion of Minute and Hour records
	*
	* @access	private
    * @param	object
    * @param	int
    * @param	int
    * @param	array
    * @param	array
    * @param	int
    * @param	string
    * @param	string
	* @return	string
	*/
  private function _processRecord($chInfo, $chAmt, $sec, $wsArray, $polArray, $wsSize, $minDate, $hourDate)
  {
	  $x = 0;
	  $updates = '';
	  $sql = '';
	  $granData = $this->now;
	  $minSQL = '';
	  $hourSQL = '';
	  $granUpd = FALSE;
	  $minUpd = FALSE;
	  $hourUpd = FALSE;
	  
	  // Loop through all of the channels
	  foreach($chInfo->result() as $chan)
	  {
	  	$updates .= 'UPDATE channel SET ';
		$cm = '';
		
	  	if(strtotime($this->now) > strtotime($this->devInfo['grandate']))
		{
			$granUpd = TRUE;
			
			// 	Generate the values needed to calculate Watts/kWh/etc.
			$vals = $this->_wattSecDiff($chan->granws, $wsArray[$x], $this->devInfo['gransec'], $sec, $x, $wsSize);
			
			// Make sure the difference in seconds is not zero
			if($vals[0] != 0)
			{
		  		if($chan->pulse == 0)
				{
					$kwh = round(($vals[1]/3600000),5);
					$watt = round($vals[1]/$vals[0]);
					
					// Check for erroneous values 
					if($kwh > 500 || $kwh < -500 || $watt > 50000 || $watt < -50000)
					{
						$granData .= ",0";
					} 
					else
					{
						$granData .= ",".$watt;
					}
				}
				else
				{
					// Pulse = number of pulses times multiplier
					$granData .= ",".round($vals[1]*$chan->pulse);
				}
			}
			else
			{
				$granData .= ",0";
			}
			
			$updates .= "granws = ".$wsArray[$x];
			$cm = ',';	
		}
		
		if(strtotime($minDate) > strtotime($this->devInfo['mindate']))
		{
			$minUpd = TRUE;
			
			// Setup SQL statement
	 		$minSQL .= "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost) 
	 					VALUES (".$chan->channel_id.",'".$minDate."',".$wsArray[$x].",".$sec;
			
			// 	Generate the values needed to calculate Watts/kWh/etc.
			$vals = $this->_wattSecDiff($chan->minws, $wsArray[$x], $this->devInfo['minsec'], $sec, $x, $wsSize);
			
			
			// Check if seconds is zero
			if($vals[0] != 0)
			{
		  		if($chan->pulse == 0)
				{
					$kwh = round(($vals[1]/3600000),5);
					$watt = round($vals[1]/$vals[0]);
					
					// Check for erroneous values
					if($kwh > 500 || $kwh < -500 || $watt > 50000 || $watt < -50000)
					{
						$minSQL .= ",0,0,0,0); ";
					} 
					else
					{
						// Add generated values to SQL statement for a normal channel
						$minSQL .= ",".$watt.",".$kwh.",
								".round(calcCost($watt/1000, $this->sysinfo),6).",
								".round(calcCost($kwh, $this->sysinfo),6)."); ";
					}
				}
				else
				{
					// Add generated values to SQL statement for pulse
					$minSQL .= ",".$vals[1]*$chan->pulse.",".round(($vals[1]/$vals[0])*3600).",0,0); ";
				}
			}
			else
			{
				$minSQL .= ",0,0,0,0); ";
			}
			
						
			$updates .= $cm." minws = ".$wsArray[$x];	
			$cm = ',';
		}
		
		if(strtotime($hourDate) > strtotime($this->devInfo['hourdate']))
		{
			$hourUpd = TRUE;
			
			// Start of Hour SQL statement
	  		$hourSQL .= "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost) VALUES 
							(".$chan->channel_id.",'".$hourDate."',".$wsArray[$x].",".$sec;
			
			// 	Generate the values needed to calculate Watts/kWh/etc.				
			$vals = $this->_wattSecDiff($chan->hourws, $wsArray[$x], $this->devInfo['hoursec'], $sec, $x, $wsSize);
			
			// Check for zero seconds
			if($vals[0] != 0)
			{
		  		if($chan->pulse == 0)
				{
					$kwh = round(($vals[1]/3600000),5);
					$watt = round($vals[1]/$vals[0]);
					
					// Check for erroneous values
					if($kwh > 500 || $kwh < -500 || $watt > 50000 || $watt < -50000)
					{
						$hourSQL .= ",0,0,0,0); ";
					} 
					else
					{
						// Add generated values to SQL statement for a normal channel
						$hourSQL .= ",".$watt.",".$kwh.",
								".round(calcCost($watt/1000, $this->sysinfo),6).",
								".round(calcCost($kwh, $this->sysinfo),6)."); ";
					}
				}
				else
				{
					// Add generated values to SQL statement for a pulse channel
					$hourSQL .= ",".$vals[1]*$chan->pulse.",".round(($vals[1]/$vals[0])*3600).",0,0); ";
				}
			}
			else
			{
				$hourSQL .= ",0,0,0,0); ";
			}
			
			$updates .= $cm." hourws = ".$wsArray[$x];
		}
		
		$updates .= " WHERE channel_id = ".$chan->channel_id."; ";
		$x++;
	  }
	  
	  
	  $updates .= "UPDATE devices SET ";
	  $cm = '';
	  
	  // If granular, update device
	  if($granUpd)
	  {
	  	$updates .= "grandate = '".$this->now."', gransec = ".$sec." ";
		$cm = ',';
		
		// Check granular data file, make sure it doesn't exceed 20 lines
		$mode = 'a';
		if(file_exists($this->devInfo['serial']))
		{
			if(count(file($this->devInfo['serial'])) > 20)
			{
			 	$this->_deleteFirst($this->devInfo['serial']);
			}
		}
		else {
			$mode = 'w';
		}
		
		// Write granular packet to real-time data file holder
		$ch = fopen($this->devInfo['serial'], $mode);
		fwrite($ch, $granData);
		fclose($ch);
	  }
	  
	  // If minutedata is updated, update the minute date and seconds
	  if($minUpd)
	  {
	  	$updates .= $cm."mindate = '".$minDate."', minsec = ".$sec." ";
		$cm = ',';
		$minSQL .= '; ';
		$sql .= $minSQL;
	  }
	  
	  
	  // If hourdata is updated, update the hour date and seconds
	  if($hourUpd)
	  {
	  	$updates .= $cm."hourdate = '".$hourDate."', hoursec = ".$sec." ";
		$hourSQL .= '; ';
		$sql .= $hourSQL;
	  }
	
	  // Check to make sure there's a proper update.
	  //
	  // Updating device early in case of another packet coming in trying to log
	  // the same date.
	  if(strlen($updates) > 20)
	  {
	  	// Finish the update packet, and send them off
	  	$updates .= "WHERE device_id = ".$this->devInfo['device_id']."; ";
		
		$sqlCmds = explode(';',$updates);
				
		$this->db->query("BEGIN;");
		
		
		for($x=0;$x<sizeof($sqlCmds);$x++)
		{
			if(strlen($sqlCmds[$x]) > 5)
			{
				$this->db->query($sqlCmds[$x]);
			}
		}
		$this->db->query("COMMIT;");
	  }
	  
	  return $sql;
  }
  
   // --------------------------------------------------------------------

   /**
	* Take in current and previous wattseconds and seconds, then calculate the difference.
	*
	* @access	private
	* @return	array
	*/
  private function _wattSecDiff($prevWS, $currWS, $prevSec, $currSec, $chnum, $wsSize)
  {
	if($prevSec != $currSec)
    {
    	// Calculate difference in seconds, rolls over at 256^3
        if($prevSec > $currSec)
        {
          $second = pow(256,3) - $prevSec + $currSec;
        }
        else
        {
          $second = $currSec - $prevSec;
        } 
        
		// Calculate difference in wattseconds, rollover varies
        if($prevWS > $currWS)
        {
          if($chnum > 2)
          {
            $wsec = pow(256,$wsSize) - $prevWS + $currWS;  // ECM-1240 AUX1-5 are 4 byte counters
          }
          else
          {
            $wsec = pow(256,5) - $prevWS + $currWS;  // GEM is all 5-byte counters
          }
        }
        else
        {
          $wsec = $currWS - $prevWS;
        } 
		
		return array($second, $wsec);
	}
	else
	{
		return array(0,0);
	}  
			
  }

   // --------------------------------------------------------------------

   /**
	* Calculate the values for a NET packet and produce the insert SQL
	*
	* @access	private
	* @return	string
	*/
  private function _processNet($net, $ws, $polws, $chid, $date, $sec, $type)
  {
    $prevWS = $this->datalog_model->GetChannel($chid);
    $prevWS = $prevWS->row_array();
	
	// Get previous values depending on date type
	if($type == 0)
	{
	  	$prevPWS = $net->minws;
		$seconds = $this->devInfo['minsec'];	
		$prevAWS = $prevWS['minws'];
	}
	else if($type == 1)
	{
		$prevPWS = $net->hourws;
		$seconds = $this->devInfo['hoursec'];	
		$prevAWS = $prevWS['hourws'];
	}
	else {
		$prevPWS = $net->granws;
		$seconds = $this->devInfo['gransec'];	  	
		$prevAWS = $prevWS['granws'];
	}
	
    if($seconds != $sec)
    {
    	 // Calculate difference in seconds, rolls over at 256^3
         if($seconds > $sec)
         {
            $second = pow(256,3) - $seconds + $sec;
         }
         else
         {
            $second = $sec - $seconds;
         } 
          
		 // Calculate difference in pol. wattseconds, rolls over at 256^5. 
		 // NET metering is restricted to CH1/CH2 on the ECM-1240
         if($prevPWS > $polws)
         {
            $polWattS = pow(256,5) - $prevPWS + $polws;
         }
         else
         {
            $polWattS = $polws - $prevPWS;
         }
         
		 
		 // Calculate difference in abs. wattseconds, rolls over at 256^5. 
		 // NET metering is restricted to CH1/CH2 on the ECM-1240
         if($prevAWS > $ws)
         {
            $wsec = pow(256,5) - $prevAWS + $ws;
         }
         else
         {
             $wsec = $ws - $prevAWS;
         }

         // Absolute is generated + used, so NET is
         // absolute minus 2 times generated
         $watt = round(($wsec-2*$polWattS) / $second);
         $kwh = round(($wsec-2*$polWattS)/3600000,5);

         $cost = round(calcCost($watt/1000, $this->sysinfo),4)*-1;
         $totCost = round(calcCost($kwh, $this->sysinfo),4)*-1;
		 
		 // Check for erroneous values
		 if($kwh > 500 || $kwh < -500 || $watt > 50000 || $watt < -50000)
		 {
			$watt = 0;
			$kwh = 0;
			$cost = 0;
			$totCost = 0;
		 }
       }

	  //Insert Minute
      if($type == 0)
      {
        return "INSERT INTO minutedata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
			  				VALUES (".$net->channel_id.",'".gmdate('Y-m-d H:i:00')."',".$polws.",".$sec.",".$watt.",".$kwh.",".$cost.",".$totCost."); ";
      }
      //Insert Hour
      else if($type == 1)
      {
        return "INSERT INTO hourdata (channel_id, date, wattsecond, seconds, watt, kwh, cost, totalcost)
			  				VALUES (".$net->channel_id.",'".gmdate('Y-m-d H:00:00')."',".$polws.",".$sec.",".$watt.",".$kwh.",".$cost.",".$totCost."); ";
      }
    }

   // --------------------------------------------------------------------

   /**
	* Generate a random colour
	*
	* @access	private
	* @return	string
	*/
	private function _random_colour()
	{
	    mt_srand((double)microtime()*1000000);
	    $c = '';
	    while(strlen($c)<6){
	        $c .= sprintf("%02X", mt_rand(0, 255));
	    }
	    return $c;
	}
	
   // --------------------------------------------------------------------

   /**
	* Delete a file if it exists.
	*
	* @access	private
	* @return	void
	*/
	private function _deleteFirst($filename) {
	  $file = file($filename);
	  $output = $file[0];
	  unset($file[0]);
	  file_put_contents($filename, $file);
	  return $output;
	}
}
?>