<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * Loginbox Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	Setup
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class Loginbox extends MX_Controller
{   
  /**
   * Constructor
   *
   * @access	public
   */
    function __construct()
    {
        parent::__construct();
        $this->load->model('sysinfo_model');
        $this->load->library('session');
    	$this->load->library('form_validation');
    	$this->load->helper('form');
    	$this->load->helper('url');
    }
	
   // --------------------------------------------------------------------

   /**
	* Display login box
	*
	* @access	public
	* @return	void
	*/
    function index()
    {
		if (!$this->session->userdata('admin'))
		{
			$data['loggedin'] = true;
		}
		else
		{
			$data['loggedin'] = false;
		}
		
       $this->load->view('loginbox', $data);  
    }
	
   // --------------------------------------------------------------------

   /**
	* Login into the settings section
	*
	* @access	public
	* @return	void
	*/
	function Login()
    {
    	$this->form_validation->set_rules('user','User Name','max_length[25]');
	  	$this->form_validation->set_rules('pass','Password','min_length[1]');
		
		if($this->form_validation->run() == FALSE)
	    {
	       $data['url'] = 'login';
	       $this->load->view('login', $data);  
		}
		else
		{
			if ($this->sysinfo_model->CheckLogin(set_value('user'), set_value('pass')))
	        {
	            $this->session->set_userdata('admin', true);
	       		echo '<script>var $ = parent.$; $().colorbox.close();</script>';
	        }
			else
			{
				redirect('/loginbox/Login');
			}
		}
    }
	
   // --------------------------------------------------------------------

   /**
	* Update login information
	*
	* @access	public
	* @return	void
	*/
	function UpdateLogin($msg = "")
	{
		
		// Kick to login if not logged in
		if (!$this->session->userdata('admin'))
		{
			redirect('/loginbox/Login');
		}
		
		$this->form_validation->set_rules('admin_user','User Name','max_length[25]|required');
	  	$this->form_validation->set_rules('admin_email','Email');
	  	$this->form_validation->set_rules('admin_pass','Password','min_length[1]|required');
	  	$this->form_validation->set_rules('admin_confpass','Password','min_length[1]|matches[admin_pass]');
		
		if($this->form_validation->run() == FALSE)
	    {
		  $zone = $this->sysinfo_model->GetInfo();
		  $zone = $zone->row_array();
		    
		  $data['tz'] = $zone['timezone'];
		  $data['msg'] =  $msg;
		  $data['locale'] = $zone['locale'];
	      $this->load->view('updateadmin', $data);  
		}
		else
		{
			$this->sysinfo_model->UpdateUser(set_value('admin_user'), set_value('admin_pass'), set_value('admin_email'));
			redirect("/loginbox/UpdateLogin/1");
		}
	}
	
   // --------------------------------------------------------------------

   /**
	* Change selected time-zone
	*
	* @access	public
	* @return	void
	*/
	function UpdateTime($msg = "")
	{
		if (!$this->session->userdata('admin'))
		{
			redirect('/loginbox/Login');
		}
		
    	$this->form_validation->set_rules('timezone', 'Timezone',  'required');
		
		if($this->form_validation->run() == FALSE)
	    {
		  $zone = $this->sysinfo_model->GetInfo();
		  $zone = $zone->row_array();
		    
		  $data['locale'] = $zone['locale'];
		  $data['tz'] = $zone['timezone'];
		  $data['msg'] =  $msg;
	      $this->load->view('updateadmin', $data);  
		}
		else
		{
	  		$this->sysinfo_model->UpdateTimeZone(set_value('timezone'));
			redirect("/loginbox/UpdateLogin/1");
		}
	}
	
   // --------------------------------------------------------------------

   /**
	* Change selected locale
	*
	* @access	public
	* @return	void
	*/	
	function UpdateLocale($msg = "")
	{
		if (!$this->session->userdata('admin'))
		{
			redirect('/loginbox/Login');
		}
		
    	$this->form_validation->set_rules('locale', 'Locale',  'required');
		
		if($this->form_validation->run() == FALSE)
	    {
		  $zone = $this->sysinfo_model->GetInfo();
		  $zone = $zone->row_array();
		    
		  $data['locale'] = $zone['locale'];
		  $data['tz'] = $zone['timezone'];
		  $data['msg'] =  $msg;
	      $this->load->view('updateadmin', $data);  
		}
		else
		{
	  		$this->sysinfo_model->UpdateLocale(set_value('locale'));
			redirect("/loginbox/UpdateLogin/1");
		}
	}

   // --------------------------------------------------------------------

   /**
	* Log-out of settings section
	*
	* @access	public
	* @return	void
	*/
    function Logout()
    {
        $this->session->unset_userdata('admin');
		redirect('/');
    }
	
   // --------------------------------------------------------------------

   /**
	* Forgot your password
	*
	* @access	public
	* @return	void
	*/
	function ForgotPassword()
    {
        $this->load->view('forgotpass');  
    }
	
   // --------------------------------------------------------------------

   /**
	* Generate and email new password
	*
	* @access	public
	* @return	void
	*/
	function ResetPass()
    {
	    $sysinfo = $this->sysinfo_model->GetEmail();
	    $sys = $sysinfo->row_array();
		
    	$newPass = rand();
    	$newPass = $newPass + rand();
		
		$this->sysinfo_model->UpdatePass($newPass);
		
		$headers = "From: mail@localhost";  
	    $body = "Your server password has been reset, please login using ".$newPass." and change your password";        
	   	$x = mail($sys['email'],"Password Reset Complete", $body, $headers);
        $this->load->view('resetpass');  
    }

}