<?php
/**
 * Dashboard-Lite
 *
 *  Data Logging and Dashbaord for Brultech Energy Monitors
 *
 * @package		Dashboard-Lite
 * @author		Brultech Team
 * @copyright	Copyright (c) 2011 - 2013, Brultech, Inc.
 * @license		btech-license.txt
 * @link		http://brultech.com
 */

// ------------------------------------------------------------------------

/**
 * PowerDisplay Controller Class
 *
 * @package	Dashboard-Lite
 * @subpackage	Controllers
 * @category	Display
 * @author	Brultech Team
 * @link	http://brultech.com
 */
class PowerDisplay extends MX_Controller {
   
  /**
   * Constructor
   *
   * @access	public
   */
  function PowerDisplay()
  {
    parent::__construct();
    $this->load->model('channels_model');
    $this->load->model('sysinfo_model');
    $this->load->model('minute_model');
    $this->load->model('device_model');
    $this->load->helper('url');
    $this->load->helper('costs');
  }
  
   // --------------------------------------------------------------------

   /**
	* Generate CSV of power values to display and refresh values in top header
	*
	* @access	public
	* @return	void
	*/
  function getMainWatts()
  {
  	$sysinfo = $this->sysinfo_model->getInfo();
    $sys = $sysinfo->row_array();
    
    setlocale(LC_MONETARY, $sys['locale']);
    $locale = localeconv(); 
    
    date_default_timezone_set($sys["timezone"]);
	
	
	// Dates needed for queries
    $today = date('M j')."th ".date('g:i:s A');
    $theDate = date("Y-m-d",time());
	$userTimezone = new DateTime($theDate, new DateTimeZone($sys['timezone']));
  	$offset = $userTimezone->getOffset();
    $now = gmdate('Y-m-d H:i:s');
	$lastLoggedBegin = gmdate('Y-m-d H:i:s', mktime(date('H'), date('i')-5, date('s'), date("m") , date("d"), date("Y")));
	$beginDate = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, date("m") , date("d")-1, date("Y")));
	$yestDate = gmdate('Y-m-d H:i:s', mktime(23, 59, 59, date("m") , date("d")-3, date("Y")));
    $yest = gmdate('Y-m-d H:i:s', mktime(date("H"), date("i"), date("s"), date("m") , date("d")-2, date("Y")));
	
	
    $mainChan = $this->channels_model->getMain();
	
	// Get current usage for Main channels
	$vals = $this->minute_model->getMainPower($beginDate, $now);
    $vals = $vals->row_array();
	
	// Get yesterday usage for Main channels
	$yestVal = $this->minute_model->getMainPower($yestDate, $yest);
    $yestVal = $yestVal->row_array();

	$devs = $this->device_model->getLastLogged($lastLoggedBegin,$now);
    $watt = 0;
	$yestWatt = 0;
	
	// Get the current watts for Main Panel
    foreach($mainChan->result() as $chan)
    {
      $powInfo = $this->minute_model->getPower($beginDate, $now, $chan->channel_id, 1);
      if($powInfo->num_rows() > 0)
      {
        $pow = $powInfo->row_array();
    
        $watt += $pow['watt'];
      }
	  
    }
	
	$bill_type = "Flat Rate";
	
	// Table to display active devices
	$serials = '<table class="center">';
	$first = TRUE;
	foreach($devs->result() as $device)
	{
		$serials .= '<tr><td>'.$device->serial.' '.date("g:i:s A", strtotime($device->grandate)+$offset).'<td><tr>';
	}
	
	$serials .= '</table>';
	
	// Get the current rate.
    $rate = calcCost(1, $sys);
	
	
	echo $vals['minwatt'].'|'.$vals['maxwatt'].'|'.$watt.'|'.number_format($rate, 3, $locale['decimal_point'], $locale['thousands_sep']);
	
	$kwhImg = '';
	
	// Comparator image to show if you've increased or decreased kWh usage compared to yesterday
	if($yestVal['kwh'] > $vals['kwh'])
	{
		$kwhImg = '<a href="#" title="'.round($yestVal['kwh'],3).' kWh used Yesterday"><img src="'.base_url().'img/greendown.png"></a>';
	}
	else {
		$kwhImg = '<a href="#" title="'.round($yestVal['kwh'],3).' kWh used Yesterday"><img src="'.base_url().'img/redup.png"></a>';	
	}
	
	echo '|'.$rate['rate_title'].'|'.$bill_type.'|'.round($vals['kwh'],2).' kWh '.$kwhImg.'|'.number_format($vals['totalcost'], 3, $locale['decimal_point'], $locale['thousands_sep']).'|'.$serials.'|'.$today.'|';
  }
}
?>